# Implementation of the variable f (SE), persistence distribution (CP), and r = probability of persisting through interval
# Parameter values are copied from Excel to R via the feedR() function in the "Evidence of Absence" workbook (module 1).
# This script file is called via the VBA function estmort() in module 1. estMort() is called from any of a number of buttons in
# the workbook, including "Estimate Mortality for Given Parameter Set" and "Plot Posterior Distribution" on the "Parameters" page
# and "Calculate Posterior and Save" from the "Edit Parameters" (EnterParms) form.

# Basic approach:
# SE and scale parameter of persistence distribution are assumed to follow beta distributions with mean provided by the user
# and with variance = [(width of 95% CI)/4]^2. The shape parameter of the persistence distribution is taken as fixed.
# SE is assumed to vary independently from CP and r

# Variables are fed into R from Excel via FeedR function in Excel module 1.
# Required variables are
# X = observed carcass count
# meanSE, minSE, maxSE = mean searcher efficiency and bounds on 95% CI
# phi = sampling coverage
# k = fractional change in SE with each successive search after a carcass has fallen
# persistence_distn = "Weibull", "Lognormal", "Exponential", or "Log-logistic" with parameters pda and pdb (alpha and beta in the User's Guide)
# 95% CI for scale parameter.
# meanCP = mean carcass persistence and bounds on 95% CI
# arrfun = "Uniform" or "Beta" carcass arrival function with parameters afa and afb
# prior.f = "Uniform", "Poisson", "Negative Binomial", "User-defined" or "Informed" prior distribution.
#    The distinction between "User-defined" and "Informed" is primarily semantic, with "Informed" referring specifically
#    to the situation where this year's prior is specifically informed by last year's posterior
# prior distribution parameters = pra and prb, which have different meanings depending on distribution
#     Uniform => pra = maximum mortality, prb = 0
#     Poisson => pra = mean = lambda, prb = 0
#     Negative Binomial => pra = r, prb = p
#     User-defined or Informed => pra = mean, prb = stdev (not used in calculations but are displayed in figures and tables)
# days = vector of sampling dates, beginning at 0 and having no repeats
# rhat = probability (mean) of carcass persisting from its fall until the next search (based on search interval of Ir)
# Ir = search interval for which r is calculated. [NOTE: It is not necessary for search interval to be constant]


# sample SE from beta distribution with mean = meanSE and sd = (maxSE-minSE)/4
mu<-meanSE; sig2<-((maxSE-minSE)/4)^2
betaa<-mu^2/sig2*(1-mu)-mu; betab<-betaa*(1/mu-1)
nt<-length(days)-1
nsim<-10000 # the number of simulation draws
tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]  #
# clip the tj array so that the probability of persisting from the end of any arrival interval to any search interval > minS (0.01)
minS<-0.01 # track carcasses only until the probability of persisting is < minS
if (persistence_distn=="Exponential"){
  qtime<-qexp(1-minS,rate=1/maxb)
} else if (persistence_distn=="Weibull"){
  qtime<-qweibull(.99,shape=pda,scale=maxb)
} else if (persistence_distn=="Log-Logistic"){
  qtime<-qllogis(.99,shape=pda,scale=maxb)
} else if (persistence_distn=="Lognormal"){
  qtime<-qlnorm(.99,meanlog=maxb, sdlog=sqrt(pda))
}
tj<-tj[days[tj[,1]]-days[tj[,2]]<=qtime,]
ntj<-length(tj[,1])

r<-numeric(nsim)
span<-max(days)
f<-rbeta(nsim,shape1=betaa,shape2=betab)
a<-numeric(nsim)+pda
# sample the persistence distribution beta parameter
if (persistence_distn=="Weibull" | persistence_distn=="Exponential"){
  b<-rlnorm(nsim,sdlog=(log(maxb)-log(minb))/(2*1.96),meanlog=log(pdb)) # from a lognormal distribution with mean = log(pdb) and sd = (log(maxb)-log(minb))/4
} else if (persistence_distn=="Lognormal"){
  b<-pmax(rnorm(nsim,mean=pdb,sd=(maxb-minb)/(2*1.96)),.1)
} else if (persistence_distn=="Log-Logistic"){
  b<-rlnorm(nsim,meanlog=log(pdb),sdlog=(log(maxb)-log(minb))/(2*1.96)) # not entirely satisfactory...doesn't look great for highly skewed distribution of b
}
# the shape parameter is assumed to be fixed and = pda.
if (prior.f=="Uniform"){
   cut_off<-pra
   prior_M<-rep(1/(pra+1),pra+1)
} else if (prior.f=="User-defined" | prior.f== "Informed"){
   cut_off<-length(prior_M)-1
  # assumes prior_M has already been read into R
}
fn.arrivals <- function(tt){ # uniform arrivals are assumed
  return(tt^0/span) 
}
#### The following section of code splits the nsim runs into several blocks to keep the array size
#### less than 15 million. If the machine does not handle array that large, then there should be an error message (how?)

# Calculate SEn
maxmiss<-max(tj[,1]-tj[,2])
powk<-k^(0:(maxmiss))
fppowk<-as.vector(apply(1-array(rep(f,each=maxmiss+1)*rep(c(0,powk[-(maxmiss+1)]),nsim),dim=c(maxmiss+1,nsim)),F='cumprod',M=2))
fpowk<-rep(f,each=maxmiss+1)*rep(powk,nsim)
SEn<-fpowk*fppowk
# Calculate integral.part: Since the arrival rates and search intervals are constant, this just needs to be calculated for each unique number of misses
tjq<-cbind(1:(1+maxmiss),rep(1,1+maxmiss))
x<-array(dim=c(dim(tjq)[1],5))                                                                         #
x[,1]<-days[tjq[,2]]                                                                                   #
x[,5]<-days[tjq[,2]+1]                                                                                 #
x[,3]<-(x[,1]+x[,5])/2                                                                                #
x[,2]<-(x[,1]+x[,3])/2                                                                                #
x[,4]<-(x[,3]+x[,5])/2
si<-days[2]
if (persistence_distn=="Exponential"){
  a<-1/b
  integral.part<-as.vector(simpson(tj=repmat(tjq,nsim,1),persistence_distn=persistence_distn,days=days[1:(maxmiss+2)],repmat(x,nsim,1),rep(a,each=maxmiss+1),b,fn.arrivals,si))
} else {
  integral.part<-as.vector(simpson(repmat(tjq,nsim,1),persistence_distn,days=days[1:(maxmiss+2)],repmat(x,nsim,1),pda,rep(b,each=maxmiss+1),fn.arrivals,si))
}
ipSEn<-SEn*integral.part
## tally how many times each number of misses occurs in the tj array
intcount<-rep(tabulate(as.vector(tj%*%c(1,-1))+1),nsim)
prob_obs<-phi*as.vector(t(array(intcount*ipSEn,dim=c(maxmiss+1,nsim)))%*%(numeric(maxmiss+1)+1))
Lpobs<-round(quantile(prob_obs,probs=.025),3)
Upobs<-round(quantile(prob_obs,probs=.975),3)
muB<-mean(prob_obs); sig2B<-var(prob_obs)
if (!pobs){
   Ba<-muB^2/sig2B*(1-muB)-muB; Bb<-Ba*(1/muB-1)
   Bab<-suppressWarnings(fitdistr(prob_obs,'beta',start=list(shape1=Ba,shape2=Bb),control=list(parscale=c(Ba,Bb)))$estimate)
   pXgM<-numeric(length(prior_M))
   Mprior<-0:(length(prior_M)-1)
   pXgM[X<=Mprior]<-choose(Mprior[X<=Mprior],X)*exp(lbeta(X+Bab[1],Mprior[X<=Mprior]-X+Bab[2])-lbeta(Bab[1],Bab[2]))
   numerator <- pXgM * prior_M
   denominator <- sum( pXgM * prior_M)
   prob_M <- numerator / denominator
   cs<-cumsum(prob_M)
   upM<-min(which(cs>.999))
   k1<-0:(length(prior_M)-1)
   pM <- list(k1[1:upM], prob_M[1:upM]/sum(prob_M[1:upM]), mean(prob_obs))
   names(pM) <- c("m","P(M=m)", "P(Observe|Arrive)")
   cs<-cumsum(pM$'P(M=m)')
   sumstat<-c(pM$"P(Observe|Arrive)",min(which(cs>=crlev)) - 1) # c(overall detection probability, mhat, credible max)
}
