# Implementation of the variable f (SE), persistence distribution (CP), and r = probability of persisting through interval
# Parameter values are copied from Excel to R via the feedR() function in the "Evidence of Absence" workbook (module 1).
# This script file is called via the VBA function estmort() in module 1. estMort() is called from any of a number of buttons in
# the workbook, including "Estimate Mortality for Given Parameter Set" and "Plot Posterior Distribution" on the "Parameters" page
# and "Calculate Posterior and Save" from the "Edit Parameters" (EnterParms) form.

# Basic approach:
# SE and scale parameter of persistence distribution are assumed to follow beta distributions with mean provided by the user
# and with variance = [(width of 95% CI)/4]^2. The shape parameter of the persistence distribution is taken as fixed.
# SE is assumed to vary independently from CP and r

# Variables are fed into R from Excel via FeedR function in Excel module 1.
# Required variables are
# X = observed carcass count
# meanSE, minSE, maxSE = mean searcher efficiency and bounds on 95% CI
# phi = sampling coverage
# k = fractional change in SE with each successive search after a carcass has fallen
# persistence_distn = "Weibull", "Lognormal", "Exponential", or "Log-logistic" with parameters pda and pdb (alpha and beta in the User's Guide)
# 95% CI for scale parameter.
# meanCP = mean carcass persistence and bounds on 95% CI
# arrfun = "Uniform" or "Beta" carcass arrival function with parameters afa and afb
# prior.f = "Uniform", "Poisson", "Negative Binomial", "User-defined" or "Informed" prior distribution.
#    The distinction between "User-defined" and "Informed" is primarily semantic, with "Informed" referring specifically
#    to the situation where this year's prior is specifically informed by last year's posterior
# prior distribution parameters = pra and prb, which have different meanings depending on distribution
#     Uniform => pra = maximum mortality, prb = 0
#     Poisson => pra = mean = lambda, prb = 0
#     Negative Binomial => pra = r, prb = p
#     User-defined or Informed => pra = mean, prb = stdev (not used in calculations but are displayed in figures and tables)
# days = vector of sampling dates, beginning at 0 and having no repeats
# rhat = probability (mean) of carcass persisting from its fall until the next search (based on search interval of Ir)
# Ir = search interval for which r is calculated. [NOTE: It is not necessary for search interval to be constant]


# sample SE from beta distribution with mean = meanSE and sd = (maxSE-minSE)/4
mu<-meanSE; sig2<-((maxSE-minSE)/4)^2
betaa<-mu^2/sig2*(1-mu)-mu; betab<-betaa*(1/mu-1)
nt<-length(days)-1
nsim<-10000 # the number of simulation draws
bsim<-min(nsim,floor(15000000/(nt^2))) # number of simulation draws the program can handle in one batch
if (bsim<nsim){
  nbat<-floor(nsim/bsim)       # number of batches that need to be run
  nsim<-bsim*nbat       # number of simulations that need to be run in addition to the blocks
} else {
  nbat<-1
}
r<-numeric(nsim)
span<-max(days)
f<-rbeta(nsim,shape1=betaa,shape2=betab)
a<-numeric(nsim)+pda
# sample the persistence distribution beta parameter 
if (persistence_distn=="Weibull" | persistence_distn=="Exponential"){
  b<-rlnorm(nsim,sdlog=(log(maxb)-log(minb))/(2*1.96),meanlog=log(pdb)) # from a lognormal distribution with mean = log(pdb) and sd = (log(maxb)-log(minb))/4
} else if (persistence_distn=="Lognormal"){
  b<-pmax(rnorm(nsim,mean=pdb,sd=(maxb-minb)/(2*1.96)),.1)
} else if (persistence_distn=="Log-Logistic"){
  b<-rlnorm(nsim,meanlog=log(pdb),sdlog=(log(maxb)-log(minb))/(2*1.96)) # not entirely satisfactory...doesn't look great for highly skewed distribution of b
}
# the shape parameter is assumed to be fixed and = pda.
if (arrfun=="Uniform"){
   parrt<-numeric(nt)+1/nt
} else if (arrfun=="Beta"){
   dug<-pbeta(days/span,shape1=afa,shape2=afb)
   parrt<-dug[2:(nt+1)]-dug[1:nt]
}
if (prior.f=="Uniform"){
   cut_off<-pra
   prior_M<-rep(1/(pra+1),pra+1)
} else if (prior.f=="User-defined" | prior.f== "Informed"){
   cut_off<-length(prior_M)-1
  # assumes prior_M has already been read into R
}
#arrfun<-"Uniform"
if (arrfun=="Uniform"){
   fn.arrivals <- function(tt){ 
      return(tt^0/span) # should this be /nt rather than /span???
   }
} else if (arrfun=="Beta"){
   fn.arrivals <- function(x){
      return(dbeta(x/span,shape1=afa,shape2=afb)/span)
   }
}
prob_obs<-numeric(nsim)
tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]  #
x<-array(dim=c(dim(tj)[1],5))                                                                         #
x[,1]<-days[tj[,2]]                                                                                   #
x[,5]<-days[tj[,2]+1]                                                                                 #
x[,3]<-(x[,1]+x[,5])/2                                                                                #
x[,2]<-(x[,1]+x[,3])/2                                                                                #
x[,4]<-(x[,3]+x[,5])/2
si<-x[,5]-x[,1]                                                                                       #
# Utility function to unravel nvec into tjvec:
tjvec.ind<-numeric(nt*(nt+1)/2)                                                                       #
beg<-1                                                                                                #
for (i in 0:(nt-1)){ #returns a set of indices so that nvec[tjvec.ind] has the desired length and format
   len<-nt-i                                                                                           #
   tjvec.ind[beg:(beg+len-1)]<-1:len                                                                   #
   beg<-beg+len                                                                                        #
}                                                                                                      #
lastday<-days[length(days)]                                                                            #
powk<-k^(0:(nt-2))                                                                                     #

#### The following section of code splits the nsim runs into several blocks to keep the array size
#### less than 15 million. If the machine does not handle array that large, then there should be an error message (how?)
for (bati in 1:nbat){
  nvec<-cbind(numeric(bsim)+1,t(apply(1-powk%o%f[(bati-1)*bsim+1:bsim],FUN="cumprod",MARGIN=2)))*f[(bati-1)*bsim+1:bsim]
  nvec<-t(t(nvec)*k^(0:(nt-1)))
   #     integral.part<-array(numeric(length(tjvec.ind)*nsim),dim=c(length(tjvec.ind),nsim))
  if (persistence_distn=="Exponential"){
     a<-1/b
  }
  integral.part<-array(dim=c(nt*(nt+1)/2,bsim))
  for (i in 1:bsim){
        integral.part[,i]<-as.vector(simpson(tj,persistence_distn,days,x,a[(bati-1)*bsim+i],b[(bati-1)*bsim+i],fn.arrivals,si))
  }
  hold<-t(nvec[,tjvec.ind])*integral.part
  M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
  ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
  mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
  junk<-array(rep(1:nt,nt),dim=c(nt,nt))
  scale.ind<-junk[lower.tri(junk,diag=T)]
  pr.t<-as.matrix(mltmat%*%(hold/parrt[scale.ind]))
  pr.t<-phi*pr.t
  prob_obs[(bati-1)*bsim+1:bsim]<-as.vector(parrt%*%pr.t/sum(parrt))
}

Lpobs<-round(quantile(prob_obs,probs=.025),3)
Upobs<-round(quantile(prob_obs,probs=.975),3)
muB<-mean(prob_obs); sig2B<-var(prob_obs)
if (!pobs){
   Ba<-muB^2/sig2B*(1-muB)-muB; Bb<-Ba*(1/muB-1)
   Bab<-suppressWarnings(fitdistr(prob_obs,'beta',start=list(shape1=Ba,shape2=Bb),control=list(parscale=c(Ba,Bb)))$estimate)
   pXgM<-numeric(length(prior_M))
   Mprior<-0:(length(prior_M)-1)
   pXgM[X<=Mprior]<-choose(Mprior[X<=Mprior],X)*exp(lbeta(X+Bab[1],Mprior[X<=Mprior]-X+Bab[2])-lbeta(Bab[1],Bab[2]))
   numerator <- pXgM * prior_M
   denominator <- sum( pXgM * prior_M)
   prob_M <- numerator / denominator
   cs<-cumsum(prob_M)
   upM<-min(which(cs>.999))
   k1<-0:(length(prior_M)-1)
   pM <- list(k1[1:upM], prob_M[1:upM]/sum(prob_M[1:upM]), mean(prob_obs))
   names(pM) <- c("m","P(M=m)", "P(Observe|Arrive)")
   cs<-cumsum(pM$'P(M=m)')
   sumstat<-c(pM$"P(Observe|Arrive)",min(which(cs>=crlev)) - 1) # c(overall detection probability, mhat, credible max)
}
