list.of.packages <- c("tensorA", "Matrix", "actuar","MASS","VGAM")
new.packages <- list.of.packages[!(list.of.packages %in% installed.packages()[,"Package"])]
if(length(new.packages)) install.packages(new.packages)
require(tensorA)# for fast calculation with multidimensional arrays (avoiding "for" loops)
require(Matrix) # sparse matrices for fast calculation and easy storage
require(actuar) # log-logistic distribution
require(MASS)
require(VGAM) # betabinomial distribution

# Common variables--used in several different functions:
# X = carcass count
# a = search coverage; value in (0, 1]
# p = searcher efficiency = prob(counting carcass | present at search); value in (0, 1]
# k = fractional change in searcher efficiency with each search
# persistence_distn = name of persistence distribution = "Weibull", "Exponential", "Lognormal", "Log-logistic"
# pda, pdb = alpha and beta parameters for persistence distribution (or, if exponential, pda = lambda = rate and pdb = irrelevant)
# fn.arrivals = actual defined arrival function (not just function name)
# afa, afb = alpha and beta parameters of arrival function (or, if uniform, irrelevant)
# prior.f = name of prior distribution = "Uniform", "Poisson", "Negative Binomial", "User-defined", "Informed"
# pra, prb = parameters for prior.f:                                                ss
#     if "Uniform", then pra = maximum value that m can conceivably have, prb = 0
#     if "Negative Binomial", then pra = r, prb = p
#     if "Poisson", then pra = lambda = rate, prb = irrelevant
#     if "User-defined" or "Informed", then pra = mean, prb = stdev (not used in calculations but shown on figs and tables)
# days = vector of search days (begins with 0, all values are positive, and no duplicates)
#dtck<- -0.01 # default tck value for plots
#setHook('plot.new', function() par(family="serif", mgp=c(2,.7,0),tck=dtck))
#setHook('plot.new', function() par(family="serif"))
#### Mest1B: Bayes estimate of posterior distribution of M for a given set of parameters
# Returns dataframe with P(M = 0:cut_off) and P(observing a carcass that falls in the study area)
Mest1B <- function(X, phi, f, k, pda, pdb, afa, afb, pra, prb, persistence_distn, fn.arrivals, prior.f, days) {
# Prior distribution is defined for P(M = 0), P(M = 1), ... P(M = m) with P(M > m) < 0.001
   if (prior.f=="Uniform") {
      cut_off<-pra
      prior_M<-rep(1/(cut_off+1), cut_off+1)
   }
   if (prior.f=="Negative Binomial"){
      cut_off<-qnbinom(.999,size=pra,prob=prb)+1
      prior_M<-dnbinom(0:cut_off, size=pra, prob=prb)
      prior_M<-prior_M/sum(prior_M)
   }
   if (prior.f=="Poisson"){
      cut_off<-qpois(.999,lambda=pra)
      prior_M<-dpois(0:cut_off,lambda=pra)
      prior_M<-prior_M/sum(prior_M)
   }
   if (prior.f=="User-defined" | prior.f=="Informed"){
   # prior_M has already been read into R
      cut_off<-length(prior_M)-1
   }

   k1 <- 0:cut_off  # total of carcasses [used for P(M = k1)]
   ## Now, get pr.1, which is P(seen in survey tt), for tt = 1, ... , nt
   nt<-length(days)-1 # number of sampling dates
   scale.init.1 <- numeric(nt)
   hold.tj <- matrix(0, nt, nt)
   # calculate probability carcass persists until sampling given sampling date
   for(tt in 1:nt){
     for(j in 1:tt){
        hold.tj[tt,j] <- prod(1-f*k^(0:(tt-j-1)))^(j<tt)*f*k^(tt-j)*fn.Psi.1(tt=tt,j=j,days=days, persistence_distn, pda, pdb)
     }
     scale.init.1[tt] <- integrate(fn.arrivals, days[tt], days[tt+1])$val
   }
   scale.init <- sum(scale.init.1)
   pr.t <- numeric(nt)
   for(tt in 1:nt){
      pr.t[tt] <- phi*sum(hold.tj[tt,] / scale.init.1[tt])
   }
   prob_obs = sum(pr.t * scale.init.1 / scale.init)
 ## now, assuming independence of carcasses, we have our probabilities via the binomial:
 probs_obs_x <- dbinom(sum(X), k1, prob_obs)
 numerator <- probs_obs_x * prior_M
 denominator <- sum( probs_obs_x * prior_M)
 prob_M <- numerator / denominator
 cs<-cumsum(prob_M)
 output.fn <- list(k1[1:min(which(cs>.999))], prob_M[1:min(which(cs>.999))]/sum(prob_M[1:min(which(cs>.999))]), prob_obs)
 names(output.fn) <- c("m","P(M=m)", "P(Observe|Arrive)")
 return(output.fn)
} ## end of function Mest1B
### Functions internal to Mest functions
fn.Psi.1 <- function(tt,j,days,persistence_distn, pda, pdb){
  if (persistence_distn=="Exponential"){
    ans <-integrate(int_Exp_Psi_0, days[j], days[j+1],tt=tt, days=days,pda=pda,subdivisions=1000)$val
  }
  if (persistence_distn=="Weibull"){
    ans <-integrate( int_Weib_Psi_0,days[j],days[j+1],tt=tt, days=days, pda = pda, pdb = pdb,subdivisions=1000)$val
  }
  if (persistence_distn=="Log-Logistic"){
    ans <- integrate( int_LLogis_Psi_0,days[j],days[j+1], tt = tt, days=days, pda= pda, pdb = pdb,subdivisions=1000)$val
  }
  if (persistence_distn=="Lognormal"){
    ans <- integrate( int_LNorm_Psi_0,days[j],days[j+1], tt = tt, days=days, pda = pda, pdb = pdb,subdivisions=1000)$val
  }
  return(ans)
}

int_Exp_Psi_0 <-  function(z,tt, days, pda){
  return(pexp(days[tt+1]-z, pda, lower.tail=FALSE)* fn.arrivals(z))
}
int_Weib_Psi_0 <- function(z,tt, days, pda, pdb){
   return(pweibull(days[tt+1]-z, shape=pda, scale = pdb, lower.tail=FALSE)* fn.arrivals(z))
}
int_LNorm_Psi_0 <- function(z,tt, days, pda, pdb){
  return(plnorm(days[tt+1]-z, meanlog=pdb, sdlog=sqrt(pda), lower.tail=FALSE)  * fn.arrivals(z))
}
int_LLogis_Psi_0 <- function(z,tt, days, pda, pdb){
  return(pllogis(days[tt+1]-z,shape=pda,scale=pdb,lower.tail=FALSE) * fn.arrivals(z))
}
#######################
plotPost<-function(vary=TRUE){
# This function plots the pmf of M, with red bars covering at least crlev and black bars covering at most (1-crlev).
# The distribution of M is passed as a vector pMemgX, with p.m.f. given as P(M = 0), P(M = 1), etc.
# Included in the graph labels are a description of the active parameter set.
# This function is called from Excel after loading the active parameter set and posterior distribution to R.
# Open graphing window
   nt<-length(days)-1
   m<-0:(length(pMemgX)-1)
   wd<-6.5; ht<-7
   win.graph(wid=wd,hei=ht)
   par(mar=c(11,4,4,.5))
# Plot posterior distribution
   maxplot<-min(length(m),min(which(cumsum(pMemgX)>.999)))
   m<-m[1:maxplot]; pMemgX<-pMemgX[1:maxplot];
   py<-c(1,1-cumsum(pMemgX[-length(pMemgX)]))
   #py<-pMemgX
   plot(m,py,xlab='m',ylab=paste("Prob( M \u2265 m | X = ", X ,")"))
#   plot(m,py,xlab='m',ylab=paste("Prob( M = m | X = ", X ,")"))
   bringToTop()
   col.use=rep("black", max(m))
   cs<-cumsum(pMemgX)
   cut.off.cr<-min(which(cs>=crlev))
   col.use[m[1]:(cut.off.cr)]="red"

   for(i in 1:length(m)){
     lines(x=rep(i-1,2), y= c(0,py[i]), col=col.use[i], lwd=3)
   }
   title("Posterior Distribution of M")
   #################
   # graph labels
   pdlab1<-paste("Carcass count: X = ",X)
   pdlab3<-paste("Coverage: a =",phi)
#   pdlab3<-pasubstitute(list(.laba,a==.phi),list(.laba=laba,.phi=phi))
   pdlab4<-paste("Sampling: I = ", round(days[nt+1]/nt,1), ", span = ",days[nt+1],sep="")
   if (!vary) {
     pdlab5<-paste("Search efficiency: p = ", meanSE,sep='')
   } else {
     pdlab5<-paste("Search efficiency: p = ", meanSE, ", 95% CI = [", minSE, ", ",maxSE, "]",sep='')
   }
   pdlab6<-paste("k = ",k)
   laba<-paste("Persistence: ", persistence_distn,sep='')
   if (persistence_distn=="Exponential"){
      pdlab8<-substitute(list(.laba,alpha==.pda),list(.laba=laba,.pda=signif(pda,3)))
   } else {
      pdlab8<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=signif(pda,3),.pdb=signif(pdb,3)))
   }
   if (vary) {
      minCPr<-rCPgab(persistence_distn,pda,minb,Ir)
      maxCPr<-rCPgab(persistence_distn,pda,maxb,Ir)
      pdlab9<-paste("CP: mean = ", round(meanCP,1), ", 95% CI = [",round(minCPr[1],1),", ",round(maxCPr[1],1),"]",sep='')
      pdlab10<-paste("r: mean = ",signif(rhat,3),", 95% CI = [",signif(minCPr[2],3),", ",signif(maxCPr[2],3),"]",sep='')
   }  else {
      pdlab9<-paste("mean CP = ", round(meanCP,1), ", r = ",signif(rhat,3),sep='')
      pdlab10<-""
   }
   laba<-paste("Arrivals: ",arrfun,sep='')
   if (arrfun=="Uniform"){
      pdlab11<-laba
   } else {
      pdlab11<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
   }
   laba<-paste("Prior: ",prior.f,sep='')
   if (prior.f=="Uniform"){
      pdlab12<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=signif(pra,3)))
   } else if (prior.f=="Negative Binomial"){
      pdlab12<-substitute(list(.laba,r==.pra,p==.prb),list(.laba=laba,.pra=signif(pra,3),.prb=signif(prb,3)))
   } else if (prior.f=="Poisson"){
      pdlab12<-substitute(list(.laba,lambda==.pra),list(.laba=laba,.pra=signif(pra,3)))
   } else if (prior.f=="User-defined"|prior.f=="Informed"){
      mu<-sum((0:(length(prior_M)-1))*prior_M)
      sig<-sqrt(sum((0:(length(prior_M)-1))^2*prior_M)-mu^2)
      pdlab12<-substitute(list(.laba,mu==.mu,sigma==.sig),list(.laba=laba,.mu=signif(mu,3),.sig=signif(sig,3)))
   }
   mtext("________________",side=1,line=3.5,adj=-.2)
   left<--max(m)*.15
   mtext(pdlab1,side=1,line=5,adj=0,at=left,cex=.8)
   mtext(pdlab3,side=1,line=6,adj=0,at=left,cex=.8)
   mtext(pdlab4,side=1,line=7,adj=0,at=left,cex=.8)
   mtext(pdlab5,side=1,line=8,adj=0,at=left,cex=.8)
   mtext(pdlab6,side=1,line=9,adj=0,at=left,cex=.8)
   mtext("________________",side=1,line=3.5,adj=1)
   top<-1
   if (vary){
     text(max(m),top,paste("Overall detection probability: g = ",signif(pobs,3), "\n 95% CI = [ ",round(Lpobs,3),", ",round(Upobs,3),"]",sep=""),adj=1,cex=.85)
   } else {
     text(max(m),top,paste("Overall detection probability: g = ",signif(pobs,3),sep=""),adj=1,cex=.85)
   }
   #text(max(m),.95*top,paste("Black bars cover less than ",(1-crlev)*100, "% of distribution",sep=''),adj=1,cex=.75)
   text(max(m),.9*top,paste("P(M \u2264 ",  m[cut.off.cr],") \u2265 ", crlev*100, "%",sep=''),adj=1,cex=.85)
   rgt=mean(range(m))
   mtext(pdlab8,side=1,line=5,adj=0,at=rgt,cex=.8)
   mtext(pdlab9,side=1,line=6,adj=0,at=rgt,cex=.8)
   mtext(pdlab10,side=1,line=7,adj=0,at=rgt,cex=.8)
   mtext(pdlab11,side=1,line=8,adj=0,at=rgt,cex=.8)
   mtext(pdlab12,side=1,line=9,adj=0,at=rgt,cex=.8)
}
# Plot prior distribution
plotPrior<-function(prior.f,pra){
   # graph of prior distribution (Uniform, Negative Binomial, Poisson--User-defined and Informed are plotted via plotUPrior
   if (prior.f=="Uniform") {
      cut_off<-pra
      prior_M<-rep(1/(cut_off+1), cut_off+1)
      ylim=c(0,1.6/(cut_off+1))
   }
   plot(0:cut_off,prior_M,xlab='m',ylab="P(M = m)",type='n',ylim=ylim)
   bringToTop()
   cs<-cumsum(prior_M)
   cut.off.95<-max(which(cs<0.95))+1
   toobig=FALSE
   if (cut_off>200) {
      toobig=TRUE
      polygon(c(0,cut.off.95,cut.off.95,0),c(rep(1/cut_off+1,2),0,0),col=2)
      polygon(c(cut.off.95,cut_off,cut_off,cut.off.95),c(rep(1/cut_off+1,2),0,0),col=1)
      lines(c(0,cut_off),rep(1/length(prior_M),2))
   }  else if (cut_off > 100) {
      lwd=1
   } else {
      lwd=3
   }
   if (!toobig) {
      for (i in 0:cut_off){
         points(i,prior_M[i+1])
         lines(c(i,i),c(0,prior_M[i+1]),lwd=lwd)
      }
   }
   title("Prior Distribution of M")
   prdlab<-prior.f
   mtext(prdlab)
}
######
plotUPrior<-function(pMem){
   wd<-7.5; ht<-5
   win.graph(wid=wd,hei=ht)
   cut_off<-length(pMem)-1
   plot(0:cut_off,pMem,xlab='m',ylab="P(M = m)",type='n')
   bringToTop()
   col.use=rep("red", length(pMem))
   cs<-cumsum(pMem)
   cut.off.95<-max(which(cs<0.95))+1
   col.use[cut.off.95:(length(pMem)+1)]="black"
   if (cut_off<=100) {
      lwd=3
   } else {
      lwd=1
   }
   for (i in 0:cut_off){
      points(i,pMem[i+1])
      lines(c(i,i),c(0,pMem[i+1]),col=col.use[i+1],lwd=lwd)
   }
   title("Prior Distribution of M")
   m<-0:cut_off
   mu<-sum(m*pMem)
   sig<-sqrt(sum(m^2*pMem)-mu^2)
   q95<-min(which(cumsum(pMem)>=0.95))-1
   prdlab<-paste("User-defined prior; 95th percentile = ", q95)
   mtext(prdlab)
   mtext("Black bars cover < 5% of distribution", cex=.8, side=1, adj=1,line=3.5)
}

######
plotPersDist<-function(persistence_distn,pda,pdb,bmax, bmin){
# persistence distribution is plotted: mean (red) and uncertainty (dotted)
   if (persistence_distn=="Exponential") {
      x<-seq(0,qexp(.99,pda),length=1000)
      y<-1-pexp(x,pda)
      rhat<-signif(1-integrate(pexp,lower=0,upper=Ir,rate=pda)$val/Ir,3)
      subt<-substitute(list(.dist,lambda==.pda,"mean CP"==.mn, .rlab,"I "==.I),list(.dist="Exponential",.pda=signif(pda,3),.mn=signif(1/pda,1),.rlab=bquote(r[I]==.(rhat)),.I=Ir))
      if (bmax!=0){
        y.low<-1-pexp(x,1/bmin)
        y.high<-1-pexp(x,1/bmax)
      }
   } else if (persistence_distn=="Weibull") {
      x<-seq(0,qweibull(.995,pda,pdb),length=1000)
      y<-1-pweibull(x,pda,pdb)
      rhat<-signif(1-integrate(pweibull,lower=0,upper=Ir,shape=pda,scale=pdb)$val/Ir,3)
      subt<-substitute(list(.dist,alpha==.pda,beta==.pdb,"mean CP"==.mn, .rlab,"I "==.I),list(.dist="Weibull",.pda=signif(pda,3),.pdb=signif(pdb,3),.mn=round(pdb*gamma(1+1/pda),1),.rlab=bquote(r[I]==.(rhat)),.I=Ir))
      if (bmax!=0){
        y.low<-1-pweibull(x,shape=pda,scale=bmin)
        y.high<-1-pweibull(x,shape=pda,scale=bmax)
      }
   } else if (persistence_distn=="Log-Logistic") {
      x<-seq(0,qllogis(.995,shape=pda,scale=pdb),length=1000)
      y<-1-pllogis(x,shape=pda,scale=pdb)
      rhat<-signif(1-integrate(pllogis,lower=0,upper=Ir,shape=pda,scale=pdb)$val/Ir,3)
      if (pda>1){
        subt<-substitute(list(.dist,alpha==.pda,beta==.pdb,"mean CP"==.mn, .rlab,"I "==.I),list(.dist="Log-Logistic",.pda=signif(pda,3),.pdb=signif(pdb,3),.mn=round(pdb*pi/pda/sin(pi/pda),1),.rlab=bquote(r[I]==.(rhat)),.I=Ir))
      } else {
        subt<-substitute(list(.dist,alpha==.pda,beta==.pdb,"mean CP = Inf", .rlab,"I "==.I),list(.dist="Log-Logistic",.pda=signif(pda,3),.pdb=signif(pdb,3),.rlab=bquote(r[I]==.(rhat)),.I=Ir))
      }
      if (bmax!=0){
        y.low<-1-pllogis(x,shape=pda,scale=bmin)
        y.high<-1-pllogis(x,shape=pda,scale=bmax)
      }

   } else if (persistence_distn=="Lognormal") {
      x<-seq(0,qlnorm(.995,meanlog=pdb, sdlog=sqrt(pda)),length=1000)
      y<-1-plnorm(x,meanlog=pdb, sdlog=sqrt(pda))
      rhat<-signif(1-integrate(plnorm,lower=0,upper=Ir,meanlog=pdb,sdlog=sqrt(pda))$val/Ir,3)
      subt<-substitute(list(.dist,alpha==.pda,beta==.pdb,"mean CP"==.mn, .rlab,"I "==.I),list(.dist="Lognormal",.pda=pda,.pdb=pdb,.mn=round(exp(pdb+pda/2),1),.rlab=bquote(r[I]==.(rhat)),.I=Ir))
      if (bmax!=0){
        y.low<-1-plnorm(x,sdlog=sqrt(pda),meanlog=bmin)
        y.high<-1-plnorm(x,sdlog=sqrt(pda),meanlog=bmax)
      }

   }
   plot(x,y,type='l',xlab="Days",ylab="Fraction of carcasses remaining",lwd=2,col=2,ylim=c(0,1),xaxs='i',yaxs='i')
   if (bmax!=0) {
     lines(x,y.low,lty=3)
     lines(x,y.high,lty=3)
   }
   title("Persistence Distribution")
   bringToTop()
   mtext(subt,cex=.8)
}
#####
pMgtA_fphi<-function(si=si,sivar=FALSE){
# pMgtA_fphi: draw a contour plot of P(M > tau | X) for a range of f values and of phi values
#
# INPUT: In addition to the common variables listed in the top of the file, this function expects the following
# parameters to have been defined globally in R:
# prior_M = prior probabilities for mortalities
# grinc = number of increments in grid to plot, i.e. there are grinc different f values for each of grinc different phi values
# tau = mortality action threshold (i.e., want M <= tau)
# p = grinc-vector of searcher efficiencies
# a = grinc-vector of search coverages
#
# OUTPUT: a contour plot of P(M > tau | X) for a ranges of f values and phi values

   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(f,phi,pMgtA,col="navy",xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Sampling Coverage (a)")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   if (k>0) { # if k > 0, full calculations must be done; if k == 0, a short-cut speeds up the calculation
# The probabilities fatalities exceeding threshold are calculated from the posterior distributions of M for the given parameter sets.
# The posterior distributions are binomial probabilities P(M = m; n = observed carcass count, p = prob_obs = P(detecting fallen carcass))
# prob_obs is estimated using the 'gen' estimator from Dail and Huso (2014)
# Several measures have been taken to streamline the calculation of prob_obs
# The heart of the calculation is hold[i,j] = P(observe carcass at search time d[i] | carcass arrives in interval (d[j-1], d[j]))
# hold[i,j] = P(persist to d[i] | arrive in (d[j-1],d[j])) * P(carcass counted at d[i] | arrive in (d[j-1],d[j])
# In simplified outline, the first factor is integral((1-persistence_distn(d[j]-t))*fn.arrivals(t), d[j-1], d[j] dt)/integral(fn.arrivals,d[j-1],d[j])
# The bounds on the integrations are (potentially long) arrays of search days;
# If there are 50 searches, the two integrals need to be calculated for every pair of search times, i.e., arrive at j, detect at i.
# Rather than calculating the integrals separately for each interval in a double loop that indexes i and j, the integral.part is
# approximated using Simpson's rule with integration bounds stored in long arrays.
      tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
      # the tj array is an array of indices for the lower diagonal of the 'hold' matrix
      x<-array(dim=c(dim(tj)[1],5)) # x is an array giving subdivisions for approximating the integral
      x[,1]<-days[tj[,2]]
      x[,5]<-days[tj[,2]+1]
      x[,3]<-(x[,1]+x[,5])/2
      x[,2]<-(x[,1]+x[,3])/2
      x[,4]<-(x[,3]+x[,5])/2
     # Utility function to unravel an nvec-vector into a tjvec-vector
     # e.g., nvec has indices 1 2 3, then tjvec = c(nvec[1], nvec[2], nvec[3], nvec[1], nvec[2], nvec[1])
      tjvec.ind<-numeric(nt*(nt+1)/2)
      beg<-1
      for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
         len<-nt-i
         tjvec.ind[beg:(beg+len-1)]<-1:len
         beg<-beg+len
      }
      lastday<-days[length(days)]
      ##
# calculation of the second factor, P(carcass counted at d[i] | arrive in (d[j-1],d[j]), is based on the observation that
# for a carcass to be counted at i, it needs to NOT be counted during a search prior to i AND it needs to be observed at i.
# Also, the detection probability for a given carcass decreases by a factor of k with each successive search.
# These assumptions are coded in the next few lines.
      if (k==1){
         nvec<-outer((1-f),0:(nt-1),"^")*f
      } else {
         if (nt>2){
            powk<-k^(0:(nt-2))
            nvec<-cbind(numeric(grinc)+1,t(apply(1-powk%o%f,FUN="cumprod",MARGIN=2)))*f
            nvec<-t(t(nvec)*k^(0:(nt-1)))
         } else if (nt==2){
            nvec<-cbind(numeric(grinc)+1,1-f)*f
            nvec<-t(t(nvec)*k^(0:(nt-1)))
         }
      }
# The hold matrix is the product of the two component probabilities...
      integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si))
      hold<-t(nvec[,tjvec.ind])*integral.part
# however, to speed up calculations, the hold matrix (which is lower diagonal) is stored as a long vector that needs to be
# recompiled into matrix form to facilitate further matrix algebra computations
      M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
      ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
      mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
      junk<-array(rep(1:nt,nt),dim=c(nt,nt))
      scale.ind<-junk[lower.tri(junk,diag=T)]
      pr.t<-as.matrix(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
      pr.t<-add.tensor(to.tensor(pr.t),to.tensor(phi),op='*') #tensor products simplify the code and speed the calculations
      scale.i<-to.tensor(parrt)
      names(scale.i)<-"I1"
      prob_obs<-as.array(einstein.tensor(scale.i,pr.t))/sum(parrt)
      if (X == 0){
         k1<-0:(length(prior_M)-1)
         probs_obs_x<-to.tensor(exp(k1 %o% log(1-prob_obs)))
      } else if (X > 0){
         k1<-0:(length(prior_M)-1)
         chpart<-to.tensor(choose(k1,X))
         names(chpart)<-'M'
         pnx<-to.tensor(exp((k1-X)%o%log(1-prob_obs)))
         names(pnx)<-c('M','f','phi')
         px<-to.tensor(prob_obs^X)
         names(px)<-c('f','phi')
         junk<-add.tensor(pnx,px,op='*')
         probs_obs_x<-add.tensor(junk,chpart,op='*')
      }
      prior_M<-as.tensor(prior_M)
      names(probs_obs_x)[1]<-"M"
      names(prior_M)<-"M"
      prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
      deno<-margin.tensor(prob_M,i="M")
      prob_M<-add.tensor(prob_M,deno,op='/')
      if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
         pMgtA<-1-margin.tensor(prob_M[1:(A+1),,],i="M")
      } else {
         pMgtA<-1-prob_M[1,,]
      }
     }  else if (k==0) {
         tj<-1:nt
         x<-array(0,dim=c(nt,5))
         x[,1]<-days[tj]
         x[,5]<-days[tj+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
         integral.part<-to.tensor(as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si)))
         fphi<-to.tensor(fphi)
         pXe0gMe1<-1-margin.tensor(add.tensor(fphi,integral.part,op='*'),i="I")
         pMem<-to.tensor(prior_M)
         kk<-to.tensor(0:(length(pMem)-1))
         if (X==0){
            pXe0gMem<-exp(einstein.tensor(log(pXe0gMe1),kk))
            pMemgXe0<-add.tensor(add.tensor(pXe0gMem,pMem,op='*'),einstein.tensor(pXe0gMem,pMem),op='/')
            if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA<-1-margin.tensor(pMemgXe0[,,1:(A+1)],i="I")
            } else {
               pMgtA<-1-pMemgXe0[,,1]
            }
         } else {
            k1<-0:(length(prior_M)-1)
            chpart<-to.tensor(choose(k1,X))
            names(chpart)<-'M'
            pnx<-to.tensor(exp((k1-X)%o%log(pXe0gMe1)))
            names(pnx)<-c('M','f','phi')
            px<-to.tensor((1-pXe0gMe1)^X)
            names(px)<-c('f','phi')
            junk<-add.tensor(pnx,px,op='*')
            probs_obs_x<-add.tensor(junk,chpart,op='*')
            prior_M<-as.tensor(prior_M)
            names(probs_obs_x)[1]<-"M"
            names(prior_M)<-"M"
            prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
            deno<-margin.tensor(prob_M,i="M")
            prob_M<-add.tensor(prob_M,deno,op='/')
            if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA<-1-margin.tensor(prob_M[1:(A+1),,],i="M")
            } else {
               pMgtA<-1-prob_M[1,,]
            }
         }
     }
   pMgtA[pMgtA<0]<-0
   image(f,phi,pMgtA,col=colorRampPalette(colr)(200),breaks=breaks,xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Sampling Coverage (a)")
   tit<-paste("P(M > ",A," | X = ",X, ")",sep="")
   if (sivar==FALSE){
      if (partial){
         mtext(tit,side=3,line=1.5,cex=1.5)
      } else {
         mtext(tit,side=3,line=2.5,cex=1.2,adj=0)
         mtext(paste("I = ",si,", span = ",span, ", k = ",k,sep=""), side=3,line=1,adj=0,cex=1)
         laba<-paste("persistence: ",persistence_distn,sep="")
         if (persistence_distn=="Exponential"){
            pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
            pdlab5<-paste("    mean CP = ",round(1/pda,2),", r = ", signif(1-integrate(pexp,lower=0,upper=si,rate=pda)$val/si,3),sep='')
         } else {
            pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
            if (persistence_distn=="Weibull"){
               pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", r = ",signif(1-integrate(pweibull,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
            } else if (persistence_distn=="Lognormal"){
               pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", r = ",signif(1-integrate(plnorm,lower=0,upper=si,meanlog=pdb,sdlog=sqrt(pda))$val/si,3),sep='')
            } else if (persistence_distn=="Log-Logistic"){
               pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", r = ",signif(1-integrate(pllogis,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
            }
         }
         mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(f)),cex=.8)
         mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(f)),cex=.8)
         laba<-paste("prior: ",prior.f,sep="")
         if (prior.f=="Uniform"){
            pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
         } else if (prior.f=="User-defined" | prior.f=="Informed"){
            prior_M<-as.vector(prior_M)
            q95<-min(which(cumsum(prior_M)>=0.95))-1
            pdlab5<-paste(laba,", 95th percentile = ",q95,sep="")
         }
         mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(f)),cex=.8)
         if (arrfun=="Uniform"){
            pdlab3<-"arrivals: Uniform"
         } else if (arrfun=="Beta"){
            laba<-paste("arrivals: Beta")
            pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
         }
         mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(f)),cex=.8)
      }
   } else {
      mtext(paste("Search interval (I) = ",round(si,1)),side=3,cex=.8)
   }
   lvl<-c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75)
   lvl<-lvl[abs(lvl/(1-crlev)-1)>.01]
   contour(f,phi,pMgtA,levels=lvl,add=T,labcex=1,col=colors()[638])
   if (crlev>=.8){
      clr<-colors()[405]
   } else {
      clr<-colors()[551]
   }
   contour(f,phi,pMgtA,levels=1-crlev,add=T,labcex=1,lwd=2,col=clr)
   box()
   bringToTop()
} # end pMgtA_fphi
pMgtA_fI<-function(){
# graph for P(exceed threshold) for arrays of f and sampling intervals. Function is similar to pMgtA_fphi
   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(f,phi,pMgtA,col="navy",xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Search Interval (I)")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   pMgtA<-array(0,dim=c(grinc,nsi))
   for (si.ind in 1:nsi){
      days<-seq(0,span,by=si[si.ind])
      nt<-length(days)-1
      if (arrfun=="Uniform"){
         parrt<-numeric(nt)+1/nt
      } else if (arrfun=="Beta"){
         dug<-pbeta(days/span,shape1=afa,shape2=afb)
         parrt<-dug[2:(nt+1)]-dug[1:nt]
      }
      if (k>0) {
         tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
         x<-array(dim=c(dim(tj)[1],5))
         x[,1]<-days[tj[,2]]
         x[,5]<-days[tj[,2]+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
        # Utility function to unravel nvec into tjvec:
         tjvec.ind<-numeric(nt*(nt+1)/2)
         beg<-1
         for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
            len<-nt-i
            tjvec.ind[beg:(beg+len-1)]<-1:len
            beg<-beg+len
         }
         lastday<-days[length(days)]
         ##
         if (k==1){
            nvec<-outer((1-f),0:(nt-1),"^")*f
         } else {
            if (nt>2){
               powk<-k^(0:(nt-2))
               nvec<-cbind(numeric(grinc)+1,t(apply(1-powk%o%f,FUN="cumprod",MARGIN=2)))*f
            } else if (nt==2) {
               nvec<-cbind(numeric(grinc)+1,(1-f))*f
            }  else if (nt==1){
               nvec<-numeric(grinc)+f
            }
            nvec<-t(t(nvec)*k^(0:(nt-1)))
         }
         integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind]))
         hold<-t(nvec[,tjvec.ind])*integral.part
         M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
         ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
         mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
         junk<-array(rep(1:nt,nt),dim=c(nt,nt))
         scale.ind<-junk[lower.tri(junk,diag=T)]
         pr.t<-as.matrix(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
         pr.t<-to.tensor(pr.t*phi)
         scale.i<-to.tensor(parrt)
         names(scale.i)<-"I1"
         prob_obs<-as.array(einstein.tensor(scale.i,pr.t))/sum(parrt)  # dimensions are correct; values do not seem unreasonable (i.e., between 0 and 1, small for small indices)
         if (X == 0){
            k1<-0:(length(prior_M)-1)
            probs_obs_x<-to.tensor(exp(k1 %o% log(1-prob_obs)))
         } else if (X > 0){
         # do something else...
            k1<-0:(length(prior_M)-1)
            chpart<-to.tensor(choose(k1,X))
            names(chpart)<-'M'
            pnx<-to.tensor(exp((k1-X)%o%log(1-prob_obs)))
            names(pnx)<-c('M','f')
            px<-to.tensor(prob_obs^X)
            names(px)<-c('f')
            junk<-add.tensor(pnx,px,op='*')
            probs_obs_x<-add.tensor(junk,chpart,op='*')
         }
         prior_M<-as.tensor(prior_M)
         names(probs_obs_x)[1]<-"M"
         names(prior_M)<-"M"
         prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
         deno<-margin.tensor(as.tensor(prob_M),i="M")
         prob_M<-add.tensor(prob_M,deno,op='/')
         if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
            pMgtA[,si.ind]<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
         } else {
            pMgtA[,si.ind]<-as.vector(1-prob_M[1,])
         }
        }  else if (k==0) {
            tj<-1:nt
            x<-array(0,dim=c(nt,5))
            x[,1]<-days[tj]
            x[,5]<-days[tj+1]
            x[,3]<-(x[,1]+x[,5])/2
            x[,2]<-(x[,1]+x[,3])/2
            x[,4]<-(x[,3]+x[,5])/2
            integral.part<-to.tensor(as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind])))
            fphi<-f*phi
            pXe0gMe1<-as.vector(1-(fphi%o%integral.part)%*%rep(1,length(integral.part)))
            pMem<-as.vector(prior_M)
            kk<-0:(length(pMem)-1)
            if (X==0){
               pXe0gMem<-outer(pXe0gMe1,kk,"^")
               pMemgXe0<-t(t(pXe0gMem)*pMem)/as.vector(pXe0gMem%*%pMem)
               if (A>=1){
                  pMgtA[,si.ind]<-1-pMemgXe0[,1:(A+1)]%*%rep(1,floor(A)+1)
               } else {
                  pMgtA[,si.ind]<-1-pMemgXe0[,1]
               }
            } else {
               k1<-0:(length(prior_M)-1)
               chpart<-to.tensor(choose(k1,X))
               names(chpart)<-'M'
               pnx<-to.tensor(exp((k1-X)%o%log(pXe0gMe1)))
               names(pnx)<-c('M','f')
               px<-to.tensor((1-pXe0gMe1)^X)
               names(px)<-c('f')
               junk<-add.tensor(pnx,px,op='*')
               probs_obs_x<-add.tensor(junk,chpart,op='*')
               prior_M<-as.tensor(prior_M)
               names(probs_obs_x)[1]<-"M"
               names(prior_M)<-"M"
               prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
               deno<-margin.tensor(prob_M,i="M")
               prob_M<-add.tensor(prob_M,deno,op='/')
               if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
                  pMgtA[,si.ind]<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
               } else {
                  pMgtA[,si.ind]<-as.vector(1-prob_M[1,])
               }
            }
        }
      }
      pMgtA[pMgtA<0]<-0
      image(f,si,pMgtA,col=colorRampPalette(colr)(200),breaks=breaks,xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Search Interval (I)")
      tit<-paste("P(M > ",A," | X = ",X, ")",sep="")
    if (partial){
       mtext(tit,side=3,line=1.5,cex=1.5)
    } else {
       mtext(tit,side=3,line=2.5,adj=0,cex=1.2)
       mtext(paste("Search coverage (a) = ",phi,", k = ",k,sep=""), side=3,line=1,adj=0,cex=1)
       laba<-paste("persistence: ",persistence_distn,sep="")
       if (persistence_distn=="Exponential"){
          pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
          pdlab5<-paste("    mean CP = ",round(1/pda,2),", median = ", signif(qexp(.5,rate=pda),2),sep='')
       } else {
          pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
          if (persistence_distn=="Weibull"){
             pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", median = ",round(qweibull(.5,pda,pdb),2),sep='')
          } else if (persistence_distn=="Lognormal"){
             pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", median = ",round(qlnorm(.5,meanlog=pdb,sdlog=sqrt(pda)),2),sep='')
          } else if (persistence_distn=="Log-Logistic"){
             pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", median = ",round(qllogis(.5,shape=pda,scale=pdb),2),sep='')
          }
       }
       mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(f)),cex=.8)
       mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(f)),cex=.8)
       laba<-paste("prior: ",prior.f,sep="")
       if (prior.f=="Uniform"){
          pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
       } else if (prior.f=="User-defined" | prior.f=="Informed"){
          prior_M<-as.vector(prior_M)
          q95<-min(which(cumsum(prior_M)>=0.95))-1
          pdlab5<-paste(laba,", 95th percentile = ",q95, sep="")
       }
       mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(f)),cex=.8)
       if (arrfun=="Uniform"){
          pdlab3<-"arrivals: Uniform"
       } else if (arrfun=="Beta"){
          laba<-paste("arrivals: Beta")
          pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
       }
      mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(f)),cex=.8)
   }
      contour(f,si,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      if (crlev>=.8){
         clr<-colors()[405]
      } else {
         clr<-colors()[551]
      }

      contour(f,si,pMgtA,levels=1-crlev,add=T,labcex=1,lwd=2,col=clr)
      box()
   #   return(pMgtA)
  bringToTop()
}
pMgtA_phiI<-function(){
   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(phi,si,pMgtA,col="navy",xlab="Search Coverage (a)",ylab="Search Interval (I)")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      #contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   pMgtA<-array(0,dim=c(grinc,nsi))
   for (si.ind in 1:nsi){
      days<-seq(0,span,by=si[si.ind])
      nt<-length(days)-1
      if (arrfun=="Uniform"){
         parrt<-numeric(nt)+1/nt
      } else if (arrfun=="Beta"){
         dug<-pbeta(days/span,shape1=afa,shape2=afb)
         parrt<-dug[2:(nt+1)]-dug[1:nt]
      }
      if (k>0) {
         tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
         x<-array(dim=c(dim(tj)[1],5))
         x[,1]<-days[tj[,2]]
         x[,5]<-days[tj[,2]+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
        # Utility function to unravel nvec into tjvec:
         tjvec.ind<-numeric(nt*(nt+1)/2)
         beg<-1
         for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
            len<-nt-i
            tjvec.ind[beg:(beg+len-1)]<-1:len
            beg<-beg+len
         }
         lastday<-days[length(days)]
         ## f is just a number
         if (k==1){
            nvec<-(1-f)^(0:(nt-1))*f
         } else {
            if (nt>=2){
               powk<-k^(0:(nt-2))
               nvec<-c(1,cumprod(1-powk*f))*f
            }  else if (nt==1){
               nvec<-f
            }
            nvec<-nvec*k^(0:(nt-1))
         }
         integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind]))
         hold<-nvec[tjvec.ind]*integral.part
         M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
         ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
         mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
         junk<-array(rep(1:nt,nt),dim=c(nt,nt))
         scale.ind<-junk[lower.tri(junk,diag=T)]
         pr.t<-as.vector(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
         pr.t<-pr.t%o%phi
         prob_obs<-as.vector(parrt%*%pr.t/sum(parrt))  # dimensions are correct; values do not seem unreasonable (i.e., between 0 and 1, small for small indices)
         if (X == 0){
            k1<-0:(length(prior_M)-1)
            probs_obs_x<-outer(1-prob_obs,k1,"^")
            prob_M<-t(t(probs_obs_x)*as.vector(prior_M))
            deno<-as.vector(prob_M%*%(numeric(dim(prob_M)[2])+1))
            prob_M<-prob_M/deno
            if (A>=1){
               pMgtA[,si.ind]<-as.vector(1-prob_M[,1:(A+1)]%*%(numeric(A+1)+1))
            } else {
               pMgtA[,si.ind]<-as.vector(1-prob_M[,1])
            }
         } else if (X > 0){ # this should be virtually identical to the fI case but with different dimension names.
         # do something else...
            k1<-0:(length(prior_M)-1)
            chpart<-to.tensor(choose(k1,X))
            names(chpart)<-'M'
            pnx<-to.tensor(exp((k1-X)%o%log(1-prob_obs)))
            names(pnx)<-c('M','phi')
            px<-to.tensor(prob_obs^X)
            names(px)<-c('phi')
            junk<-add.tensor(pnx,px,op='*')
            probs_obs_x<-add.tensor(junk,chpart,op='*')
            prior_M<-as.tensor(prior_M)
            names(probs_obs_x)[1]<-"M"
            names(prior_M)<-"M"
            prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
            deno<-margin.tensor(as.tensor(prob_M),i="M")
            prob_M<-add.tensor(prob_M,deno,op='/')
            if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA[,si.ind]<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
            } else {
               pMgtA[,si.ind]<-as.vector(1-prob_M[1,,])
            }
         }
        }  else if (k==0) {
            tj<-1:nt
            x<-array(0,dim=c(nt,5))
            x[,1]<-days[tj]
            x[,5]<-days[tj+1]
            x[,3]<-(x[,1]+x[,5])/2
            x[,2]<-(x[,1]+x[,3])/2
            x[,4]<-(x[,3]+x[,5])/2
            integral.part<-to.tensor(as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind])))
            fphi<-f*phi
            pXe0gMe1<-as.vector(1-(fphi%o%integral.part)%*%rep(1,length(integral.part)))
            pMem<-as.vector(prior_M)
            kk<-0:(length(pMem)-1)
            if (X==0) {
               pXe0gMem<-outer(pXe0gMe1,kk,"^")
               pMemgXe0<-t(t(pXe0gMem)*pMem)/as.vector(pXe0gMem%*%pMem)
               if (A>=1){
                  pMgtA[,si.ind]<-1-pMemgXe0[,1:(A+1)]%*%rep(1,floor(A)+1)
               } else {
                  pMgtA[,si.ind]<-1-pMemgXe0[,1]
               }
            } else { # X > 0
               k1<-0:(length(prior_M)-1)
               chpart<-to.tensor(choose(k1,X))
               names(chpart)<-'M'
               pnx<-to.tensor(exp((k1-X)%o%log(pXe0gMe1)))
               names(pnx)<-c('M','phi')
               px<-to.tensor((1-pXe0gMe1)^X)
               names(px)<-c('phi')
               junk<-add.tensor(pnx,px,op='*')
               probs_obs_x<-add.tensor(junk,chpart,op='*')
               prior_M<-as.tensor(prior_M)
               names(probs_obs_x)[1]<-"M"
               names(prior_M)<-"M"
               prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
               deno<-margin.tensor(prob_M,i="M")
               prob_M<-add.tensor(prob_M,deno,op='/')
               if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
                  pMgtA[,si.ind]<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
               } else {
                  pMgtA[,si.ind]<-as.vector(1-prob_M[1,])
               }
            }
        }
      }
      pMgtA[pMgtA<0]<-0
#      pMgtA[is.na(pMgtA)]<-1
      image(phi,si,pMgtA,col=colorRampPalette(colr)(200),breaks=breaks,xlab="Search coverage (a)\nP(carcass lands in searched area)",ylab="Search Interval (I)")
      contour(phi,si,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      if (crlev>=.8){
         clr<-colors()[405]
      } else {
         clr<-colors()[551]
      }
      contour(phi,si,pMgtA,levels=1-crlev,add=T,labcex=1,lwd=2,col=clr)
      tit<-paste("P(M > ",A," | X = ",X, ")",sep="")
    if (partial){
       mtext(tit,side=3,line=1.5,cex=1.5)
    } else {
       mtext(tit,side=3,line=2.5,adj=0,cex=1.2)
       mtext(paste("Searcher efficiency (p) = ",f,", k = ",k,sep=""), side=3,line=1,adj=0,cex=1)
       laba<-paste("persistence: ",persistence_distn,sep="")
       if (persistence_distn=="Exponential"){
          pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
          pdlab5<-paste("    mean CP = ",round(1/pda,2),", median = ", round(qexp(.5,rate=pda),2),sep='')
       } else {
          pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
          if (persistence_distn=="Weibull"){
             pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", median = ",round(qweibull(.5,pda,pdb),2),sep='')
          } else if (persistence_distn=="Lognormal"){
             pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", median = ",round(qlnorm(.5,meanlog=pdb,sdlog=sqrt(pda)),2),sep='')
          } else if (persistence_distn=="Log-Logistic"){
             pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", median = ",round(qllogis(.5,shape=pda,scale=pdb),2),sep='')
          }
       }
       mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(phi)),cex=.8)
       mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(phi)),cex=.8)
       laba<-paste("prior: ",prior.f,sep="")
       if (prior.f=="Uniform"){
          pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
       } else if (prior.f=="User-defined" | prior.f=="Informed"){
          q95<-min(which(cumsum(prior_M)>=0.95))-1
          pdlab5<-paste(laba,", 95th percentile = ",q95, sep="")
       }
       mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(phi)),cex=.8)
       if (arrfun=="Uniform"){
          pdlab3<-"arrivals: Uniform"
       } else if (arrfun=="Beta"){
          laba<-paste("arrivals: Beta")
          pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
       }
      mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(phi)),cex=.8)
   }
      box()
   #   return(pMgtA)
   bringToTop()
}
pMgtA_I<-function(){
   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(phi,si,pMgtA,col="navy",xlab="Search Coverage (a)",ylab="Search Interval (I)")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      #contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   pMgtA<-numeric(nsi)
   for (si.ind in 1:nsi){
      days<-seq(0,span,by=si[si.ind])
      nt<-length(days)-1
      if (arrfun=="Uniform"){
         parrt<-numeric(nt)+1/nt
      } else if (arrfun=="Beta"){
         dug<-pbeta(days/span,shape1=afa,shape2=afb)
         parrt<-dug[2:(nt+1)]-dug[1:nt]
      }
      if (k>0) {
         tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
         x<-array(dim=c(dim(tj)[1],5))
         x[,1]<-days[tj[,2]]
         x[,5]<-days[tj[,2]+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
        # Utility function to unravel nvec into tjvec:
         tjvec.ind<-numeric(nt*(nt+1)/2)
         beg<-1
         for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
            len<-nt-i
            tjvec.ind[beg:(beg+len-1)]<-1:len
            beg<-beg+len
         }
         lastday<-days[length(days)]
         ## f is just a number
         if (k==1){
            nvec<-(1-f)^(0:(nt-1))*f
         } else {
            if (nt>=2){
               powk<-k^(0:(nt-2))
               nvec<-c(1,cumprod(1-powk*f))*f
            }  else if (nt==1){
               nvec<-f
            }
            nvec<-nvec*k^(0:(nt-1))
         }
         integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind]))
         hold<-nvec[tjvec.ind]*integral.part
         M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
         ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
         mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
         junk<-array(rep(1:nt,nt),dim=c(nt,nt))
         scale.ind<-junk[lower.tri(junk,diag=T)]
         pr.t<-as.vector(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
         pr.t<-pr.t*phi
         prob_obs<-as.vector(parrt%*%pr.t/sum(parrt))  # dimensions are correct; values do not seem unreasonable (i.e., between 0 and 1, small for small indices)
         if (X == 0){
            k1<-0:(length(prior_M)-1)
            probs_obs_x<-outer(1-prob_obs,k1,"^")
            prob_M<-t(t(probs_obs_x)*prior_M)
            deno<-as.vector(prob_M%*%(numeric(dim(prob_M)[2])+1))
            prob_M<-prob_M/deno
            if (A>=1){
               pMgtA[si.ind]<-as.vector(1-prob_M[1:(A+1)]%*%(numeric(A+1)+1))
            } else {
               pMgtA[si.ind]<-as.vector(1-prob_M[1])
            }
         } else if (X > 0){
            k1<-0:(length(prior_M)-1)
            chpart<-choose(k1,X)
            pnx<-(1-prob_obs)^(k1-X)
            px<-prob_obs^X
            junk<-px*pnx
            probs_obs_x<-junk*chpart
            prob_M<-probs_obs_x*prior_M
            deno<-sum(prob_M)###$
            prob_M<-prob_M/deno
            if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA[si.ind]<-1-sum(prob_M[1:(A+1)])
            } else {
               pMgtA[si.ind]<-1-prob_M[1]
            }
          }
        } else if (k==0) {
            tj<-1:nt
            x<-array(0,dim=c(nt,5))
            x[,1]<-days[tj]
            x[,5]<-days[tj+1]
            x[,3]<-(x[,1]+x[,5])/2
            x[,2]<-(x[,1]+x[,3])/2
            x[,4]<-(x[,3]+x[,5])/2
#            integral.part<-to.tensor(as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind])))
            integral.part<-as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si[si.ind]))
            fphi<-f*phi
            pXe0gMe1<-as.vector(1-(fphi%o%integral.part)%*%rep(1,length(integral.part)))
            pMem<-as.vector(prior_M)
            kk<-0:(length(pMem)-1)
            if (X==0){
               pXe0gMem<-outer(pXe0gMe1,kk,"^")
               pMemgXe0<-t(t(pXe0gMem)*pMem)/as.vector(pXe0gMem%*%pMem)
               if (A>=1){
                  pMgtA[si.ind]<-1-pMemgXe0[,1:(A+1)]%*%rep(1,floor(A)+1)
               } else {
                  pMgtA[si.ind]<-1-pMemgXe0[,1]
               }
           } else {
              k1<-0:(length(prior_M)-1)
              chpart<-choose(k1,X)
              pnx<-(1-prob_obs)^(k1-X)
              px<-prob_obs^X
              junk<-px*pnx
              probs_obs_x<-junk*chpart
              prob_M<-probs_obs_x*prior_M
              deno<-sum(prob_M)###$
              prob_M<-prob_M/deno
              if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA[si.ind]<-1-sum(prob_M[1:(A+1)])
             } else {
               pMgtA[si.ind]<-1-prob_M[1]
             }
          }
      }
   }
      pMgtA[pMgtA<0]<-0
      plot(si,pMgtA,xlab=paste("Search interval over a ",span," day span",sep=""),ylab=paste("P(M > ",A," | X = ",X, ")",sep=""),type='l')
      bringToTop()
   if (partial){
      tit<-"Probability of exceeding threshold vs. search interval"
      mtext(tit,side=3,line=1.7,cex=1.5)
      mtext(substitute(list(p==.f,a==.phi, k==.k),list(.f=f,.phi=phi,.k=k)),side=3,line=.2)
   } else {
      tit<-"Probability of exceeding threshold\n vs. search interval"
      mtext(tit,side=3,line=2,adj=0,cex=1)
      mtext(substitute(list(p==.f,a==.phi,k==.k),list(.f=f,.phi=phi,.k=k)),side=3,line=.2,adj=0)
      laba<-paste("persistence: ",persistence_distn,sep="")
      if (persistence_distn=="Exponential"){
         pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
         pdlab5<-paste("    mean CP = ",round(1/pda,2),", median = ", round(qexp(.5,rate=pda),2),sep='')
      } else {
         pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
         if (persistence_distn=="Weibull"){
            pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", median = ",round(qweibull(.5,pda,pdb),2),sep='')
         } else if (persistence_distn=="Lognormal"){
             pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", median = ",round(qlnorm(.5,meanlog=pdb,sdlog=sqrt(pda)),2),sep='')
         } else if (persistence_distn=="Log-Logistic"){
            pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", median = ",round(qllogis(.5,shape=pda,scale=pdb),2),sep='')
         }
      }
      mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(si)),cex=.8)
      mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(si)),cex=.8)
      laba<-paste("prior: ",prior.f,sep="")
      if (prior.f=="Uniform"){
         pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
      } else if (prior.f=="User-defined" | prior.f=="Informed"){
          q95<-min(which(cumsum(prior_M)>=0.95))-1
          pdlab5<-paste(laba,", 95th percentile = ",q95, sep="")
      }
      mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(si)),cex=.8)
      if (arrfun=="Uniform"){
         pdlab3<-"arrivals: Uniform"
      } else if (arrfun=="Beta"){
         laba<-paste("arrivals: Beta")
         pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
      }
      mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(si)),cex=.8)
   }
      box()
   #   return(pMgtA)
   bringToTop()
}
pMgtA_phi<-function(){
   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(f,phi,pMgtA,col="navy",xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Sampling Coverage")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   if (k>0) {
      tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
      x<-array(dim=c(dim(tj)[1],5))
      x[,1]<-days[tj[,2]]
      x[,5]<-days[tj[,2]+1]
      x[,3]<-(x[,1]+x[,5])/2
      x[,2]<-(x[,1]+x[,3])/2
      x[,4]<-(x[,3]+x[,5])/2
     # Utility function to unravel nvec into tjvec:
      tjvec.ind<-numeric(nt*(nt+1)/2)
      beg<-1
      for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
         len<-nt-i
         tjvec.ind[beg:(beg+len-1)]<-1:len
         beg<-beg+len
      }
      lastday<-days[length(days)]
      ##
      if (k==1){
         nvec<-(1-f)^(0:(nt-1))*f
      } else {
         if (nt>=2){
            powk<-k^(0:(nt-2))
            nvec<-c(1,cumprod(1-powk*f))*f
         }  else if (nt==1){
            nvec<-f
         }
         nvec<-nvec*k^(0:(nt-1))
      }
      integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si))
      hold<-nvec[tjvec.ind]*integral.part
      M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
      ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
      mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
      junk<-array(rep(1:nt,nt),dim=c(nt,nt))
      scale.ind<-junk[lower.tri(junk,diag=T)]
      pr.t<-as.vector(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
      pr.t<-phi%o%pr.t #dimensions are correct; values are correct
      prob_obs<-as.vector(pr.t%*%parrt/sum(parrt))  # dimensions are correct; values do not seem unreasonable (i.e., between 0 and 1, small for small indices)
      if (X == 0){
            k1<-0:(length(prior_M)-1)
            probs_obs_x<-outer(1-prob_obs,k1,"^")
            prob_M<-t(t(probs_obs_x)*as.vector(prior_M))
            deno<-as.vector(prob_M%*%(numeric(dim(prob_M)[2])+1))
            prob_M<-prob_M/deno
            if (A>=1){
               pMgtA<-as.vector(1-prob_M[,1:(A+1)]%*%(numeric(A+1)+1))
            } else {
               pMgtA<-as.vector(1-prob_M[,1])
            }
         } else if (X > 0){ # this should be virtually identical to the phiI case but with different dimension names.
         # do something else...
            k1<-0:(length(prior_M)-1)
            chpart<-to.tensor(choose(k1,X))
            names(chpart)<-'M'
            pnx<-to.tensor(exp((k1-X)%o%log(1-prob_obs)))
            names(pnx)<-c('M','phi')
            px<-to.tensor(prob_obs^X)
            names(px)<-c('phi')
            junk<-add.tensor(pnx,px,op='*')
            probs_obs_x<-add.tensor(junk,chpart,op='*')
            prior_M<-as.tensor(prior_M)
            names(probs_obs_x)[1]<-"M"
            names(prior_M)<-"M"
            prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
            deno<-margin.tensor(as.tensor(prob_M),i="M")
            prob_M<-add.tensor(prob_M,deno,op='/')
            if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
               pMgtA<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
            } else {
               pMgtA<-as.vector(1-prob_M[1,,])
            }
         }
     } else if (k==0) {
         tj<-1:nt
         x<-array(0,dim=c(nt,5))
         x[,1]<-days[tj]
         x[,5]<-days[tj+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
         integral.part<-as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si))
         fphi<-f*phi
         pXe0gMe1<-as.vector(1-(fphi%o%integral.part)%*%(numeric(nt)+1))
         pMem<-prior_M
         kk<-0:(length(pMem)-1)
         pXe0gMem<-outer(pXe0gMe1,kk,"^")
         pMemgXe0<-t(t(pXe0gMem)*pMem)/as.vector(pXe0gMem%*%pMem)
         if (A>=1){
            pMgtA<-1-pMemgXe0[,1:(A+1)]%*%rep(1,floor(A)+1)
         } else {
            pMgtA<-1-pMemgXe0[1,]
         }
     }
   pMgtA[pMgtA<0]<-0
   plot(phi,pMgtA,xlab = expression(paste("Search Coverage (", phi,")")),ylab=paste0("P(M > ",A,"| X = ",X,")"),type='l')
   bringToTop()
   if (partial){
      tit<-"Probability of exceeding threshold vs. search coverage"
      mtext(tit,side=3,line=1.5,cex=1.5)
      mtext(paste("f = ",f, ", I = ", si, ", span = ", days[length(days)],sep=""))
   } else {
      tit<-"Probability of exceeding threshold\n vs. search coverage"
      mtext(tit,side=3,line=2,adj=0,cex=1)
      mtext(side=3,text=paste("f = ",f,", I = ", si, ", span = ",days[length(days)], ", k = ",k),adj=0)
      laba<-paste("persistence: ",persistence_distn,sep="")
      if (persistence_distn=="Exponential"){
         pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
         pdlab5<-paste("    mean CP = ",round(1/pda,2),", r = ", signif(1-integrate(pexp,lower=0,upper=si,rate=pda)$val/si,3),sep='')
      } else {
         pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
         if (persistence_distn=="Weibull"){
            pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", r = ",signif(1-integrate(pweibull,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
         } else if (persistence_distn=="Lognormal"){
            pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", r = ",signif(1-integrate(plnorm,lower=0,upper=si,meanlog=pdb,sdlog=sqrt(pda))$val/si,3),sep='')
         } else if (persistence_distn=="Log-Logistic"){
            pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", r = ",signif(1-integrate(pllogis,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
         }
      }
      mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(phi)),cex=.8)
      mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(phi)),cex=.8)
      laba<-paste("prior: ",prior.f,sep="")
      if (prior.f=="Uniform"){
         pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
      } else if (prior.f=="User-defined" | prior.f=="Informed"){
          q95<-min(which(cumsum(prior_M)>=0.95))-1
          pdlab5<-paste(laba,", 95th percentile = ",q95, sep="")
      }
      mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(phi)),cex=.8)
      if (arrfun=="Uniform"){
         pdlab3<-"arrivals: Uniform"
      } else if (arrfun=="Beta"){
         laba<-paste("arrivals: Beta")
         pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
      }
      mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(phi)),cex=.8)
   }
   bringToTop()
}
pMgtA_f<-function(){
   if (A>=length(prior_M)){ #no chance mortality exceeds what prior says is possible
      pMgtA<-array(0,dim=c(grinc,grinc))
      image(f,phi,pMgtA,col="navy",xlab="Searcher efficiency (p)\nP(observing carcass | carcass present)",ylab="Sampling Coverage")
      tit<-paste("P(M > ",A," | X = ",X, ") = 0",sep="")
      mtext(tit,side=3,line=1.5,cex=1.5)
      contour(f,phi,pMgtA,levels=c(0.001,0.01,0.025,0.05,0.1,0.25,0.5,.75),add=T,labcex=1,col=colors()[638])
      box()
      return()
   }
   if (k>0) {
      tj<-array(dim=c(nt*nt,2)); tj[,1]<-rep(1:nt,nt); tj[,2]<-sort(rep(1:nt,nt)); tj<-tj[tj[,1]>=tj[,2],]
      x<-array(dim=c(dim(tj)[1],5))
      x[,1]<-days[tj[,2]]
      x[,5]<-days[tj[,2]+1]
      x[,3]<-(x[,1]+x[,5])/2
      x[,2]<-(x[,1]+x[,3])/2
      x[,4]<-(x[,3]+x[,5])/2
     # Utility function to unravel nvec into tjvec:
      tjvec.ind<-numeric(nt*(nt+1)/2)
      beg<-1
      for (i in 0:(nt-1)){ #returns as set of indices so that nvec[tjvec.ind] has the desired length and format
         len<-nt-i
         tjvec.ind[beg:(beg+len-1)]<-1:len
         beg<-beg+len
      }
      lastday<-days[length(days)]
      ##
      if (k==1){
         nvec<-outer((1-f),0:(nt-1),"^")*f
      } else {
         powk<-k^(0:(nt-2))
         nvec<-cbind(numeric(grinc)+1,t(apply(1-powk%o%f,FUN="cumprod",MARGIN=2)))*f
         nvec<-t(t(nvec)*k^(0:(nt-1)))
      }
      integral.part<- as.vector(simpson(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si))
      hold<-t(nvec[,tjvec.ind])*integral.part
      M.ind<-array(0,dim=c(nt,nt)); M.ind[lower.tri(M.ind,diag=T)]<-1:(nt*(nt+1)/2); M.ind<-t(M.ind)
      ra<-rep.int(1,nt*(nt+1)/2); ja<-M.ind[M.ind>0]; ia<-(1:(nt+1))*(0:nt)/2
      mltmat<-sparseMatrix(j=ja,p=ia,x=ra)
      junk<-array(rep(1:nt,nt),dim=c(nt,nt))
      scale.ind<-junk[lower.tri(junk,diag=T)]
      pr.t<-as.matrix(mltmat%*%(hold/parrt[scale.ind]))#=(M/parrt)  # This is correct, with columns indexed to values of f; phi has not been added to the end yet.
      pr.t<-phi*pr.t #dimensions are correct; values are correct
      prob_obs<-as.vector(parrt%*%pr.t/sum(parrt))  # dimensions are correct; values do not seem unreasonable (i.e., between 0 and 1, small for small indices)
      if (X == 0){
         k1<-0:(length(prior_M)-1)
         probs_obs_x<-outer(1-prob_obs,k1,"^")
      } else if (X > 0){
      # do something else...
         k1<-0:(length(prior_M)-1)
         chpart<-choose(k1,X)
         pnx<-outer(1-prob_obs,k1-X,'^')
         px<-prob_obs^X
         junk<-pnx*px
         probs_obs_x<-t(t(junk)*chpart)
      }
      prob_M<-t(t(probs_obs_x)*prior_M)
      deno<-as.vector(prob_M%*%rep(1,dim(prob_M)[2]))
      prob_M<-prob_M/deno
      if (A>=1){
         pMgtA<-1-prob_M[,1:(A+1)]%*%(numeric(A+1)+1)
      } else {
         pMgtA<-1-prob_M[,1]
      }
   }  else if (k==0) {
         tj<-1:nt
         x<-array(0,dim=c(nt,5))
         x[,1]<-days[tj]
         x[,5]<-days[tj+1]
         x[,3]<-(x[,1]+x[,5])/2
         x[,2]<-(x[,1]+x[,3])/2
         x[,4]<-(x[,3]+x[,5])/2
         integral.part<-as.vector(simpson0(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si))
         fphi<-f*phi
         pXe0gMe1<-as.vector(1-(fphi%o%integral.part)%*%(numeric(nt)+1))
         pMem<-prior_M
         kk<-0:(length(pMem)-1)
         if (X==0){
            pXe0gMem<-outer(pXe0gMe1,kk,"^")
            pMemgXe0<-t(t(pXe0gMem)*pMem)/as.vector(pXe0gMem%*%pMem)
            if (A>=1){
               pMgtA<-1-pMemgXe0[,1:(A+1)]%*%rep(1,floor(A)+1)
            } else {
               pMgtA<-1-pMemgXe0[1,]
            }
###-> new
         } else { # X > 1
               k1<-0:(length(prior_M)-1)
               chpart<-to.tensor(choose(k1,X))
               names(chpart)<-'M'
               pnx<-to.tensor(exp((k1-X)%o%log(pXe0gMe1)))
               names(pnx)<-c('M','f')
               px<-to.tensor((1-pXe0gMe1)^X)
               names(px)<-c('f')
               junk<-add.tensor(pnx,px,op='*')
               probs_obs_x<-add.tensor(junk,chpart,op='*')
               prior_M<-as.tensor(prior_M)
               names(probs_obs_x)[1]<-"M"
               names(prior_M)<-"M"
               prob_M<-add.tensor(probs_obs_x,prior_M,op='*')
               deno<-margin.tensor(prob_M,i="M")
               prob_M<-add.tensor(prob_M,deno,op='/')
               if (A>=1){ #margin.tensor only works for summing if there are at least two pieces to sum
                  pMgtA<-as.vector(1-margin.tensor(prob_M[1:(A+1),],i="M"))
               } else {
                  pMgtA<-as.vector(1-prob_M[1,])
               }
            }
###<-new
     }
   pMgtA[pMgtA<0]<-0
   plot(f,pMgtA,xlab = "Searcher efficiency (p)",ylab=paste0("P(M > ",A,"| X = ",X,")"),type='l')
   bringToTop()
   if (partial){
      tit<-"Probability of exceeding threshold vs. searcher efficiency"
      mtext(tit,side=3,line=1.5,cex=1.5)
      mtext(substitute(list(a == .phi, I ==.I,"span"==.span),list(.phi=phi,.I=si,.span=days[length(days)])))
   } else {
      tit<-"Probability of exceeding threshold\n vs. searcher efficiency"
      mtext(tit,side=3,line=2,adj=0,cex=1)
      mtext(substitute(list(a == .phi, I ==.I,"span"==.span, k==.k),list(.phi=phi,.I=si,.span=days[length(days)],.k=k)),adj=0)
      laba<-paste("persistence: ",persistence_distn,sep="")
      if (persistence_distn=="Exponential"){
         pdlab4<-substitute(list(.laba,lambda==.pda),list(.laba=laba,.pda=pda))
         pdlab5<-paste("    mean CP = ",round(1/pda,2),", r = ", signif(1-integrate(pexp,lower=0,upper=si,rate=pda)$val/si,3),sep='')
      } else {
         pdlab4<-substitute(list(.laba,alpha==.pda,beta==.pdb),list(.laba=laba,.pda=pda,.pdb=pdb))
         if (persistence_distn=="Weibull"){
            pdlab5<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", r = ",signif(1-integrate(pweibull,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
         } else if (persistence_distn=="Lognormal"){
            pdlab5<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", r = ",signif(1-integrate(plnorm,lower=0,upper=si,meanlog=pdb,sdlog=sqrt(pda))$val/si,3),sep='')
         } else if (persistence_distn=="Log-Logistic"){
            pdlab5<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", r = ",signif(1-integrate(pllogis,lower=0,upper=si,shape=pda,scale=pdb)$val/si,3),sep='')
         }
      }
      mtext(pdlab4,side=3,line=3,adj=0,at=mean(range(f)),cex=.8)
      mtext(pdlab5,side=3,line=2.3,adj=0,at=mean(range(f)),cex=.8)
      laba<-paste("prior: ",prior.f,sep="")
      if (prior.f=="Uniform"){
         pdlab5<-substitute(list(.laba,mmax==.pra),list(.laba=laba,.pra=pra))
      } else if (prior.f=="User-defined" | prior.f=="Informed"){
          q95<-min(which(cumsum(prior_M)>=0.95))-1
          pdlab5<-paste(laba,", 95th percentile = ",q95, sep="")
      }
      mtext(pdlab5,side=3,line=1.3,adj=0,at=mean(range(f)),cex=.8)
      if (arrfun=="Uniform"){
         pdlab3<-"arrivals: Uniform"
      } else if (arrfun=="Beta"){
         laba<-paste("arrivals: Beta")
         pdlab3<-substitute(list(.laba,alpha==.afa,beta==.afb),list(.laba=laba,.afa=afa,.afb=afb))
      }
      mtext(pdlab3,side=3,line=.3,adj=0,at=mean(range(f)),cex=.8)
   }
   bringToTop()
}

simpson<-function(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si) { #tj is a, b; n is 4
   if (persistence_distn == "Weibull") {
      return((pweibull(days[tj[,1]+1]-x, shape=pda, scale = pdb, lower.tail=FALSE)* fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
   } else if (persistence_distn == "Lognormal") {
      return((plnorm(days[tj[,1]+1]-x, meanlog=pdb, sdlog=sqrt(pda), lower.tail=FALSE)  * fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
   } else if (persistence_distn == "Log-Logistic") {
      x1 <- days[tj[,1]+1]-x
      return((pllogis(days[tj[,1]+1]-x,shape=pda,scale=pdb,lower.tail=FALSE) * fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
   } else if (persistence_distn == "Exponential") {
      return((pexp(days[tj[,1]+1]-x, pda, lower.tail=FALSE)* fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
   } else return(0)
}
simpson0<-function(tj,persistence_distn,days,x,pda,pdb,fn.arrivals,si) { #tj is a, b; n is 4
            if (persistence_distn == "Weibull") {
              return((pweibull(days[tj+1]-x, shape=pda, scale = pdb, lower.tail=FALSE) * fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
            } else if (persistence_distn == "Lognormal") {
               return((plnorm(days[tj+1]-x, meanlog=pdb, sdlog=sqrt(pda), lower.tail=FALSE) * fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
            } else if (persistence_distn == "Log-Logistic") {
               x1 <- days[tj[,1]+1]-x
               return((pllogis(days[tj[,1]+1]-x,shape=pda,scale=pdb,lower.tail=FALSE) * fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
            } else if (persistence_distn == "Exponential") {
               return((pexp(days[tj+1]-x, pda, lower.tail=FALSE)* fn.arrivals(x))%*%c(1,4,2,4,1)*si/12)
            } else return(0)
}
prcalc<-function(prior.f,pra){
   if (prior.f=="Uniform") {
      cut_off<-pra
      return(cbind(0:cut_off,rep(1/(cut_off+1), cut_off+1)))
   }
}
lnorm.aopt<-function(a,b,t95){
   (t95-qlnorm(.95,meanlog=b,sdlog=sqrt(a)))^2
}
calc.Px1px2<-function(X1,X2){
   mmax1<-min(which(cumsum(X1)>=.9999999))-1
   mmax2<-min(which(cumsum(X2)>=.9999999))-1
   x1<-rep(0:mmax1,mmax2+1)
   x2<-sort(rep(0:mmax2,mmax1+1))
   x1px2<-x1+x2
   Px1x2<-X1[x1+1]*X2[x2+1]
   mmax<-mmax1+mmax2
   Px1px2em<-numeric(mmax+1)
   for (m in 0:mmax){
      Px1px2em[m+1]<-sum(Px1x2[x1px2==m])
   }
   return(Px1px2em)
}
new.prior<-function(old.post){
   m<-0:(length(old.post)-1)
   mu<-sum(m*old.post)
   sig2<-sum(m^2*old.post)-mu^2
   if (mu<sig2){
      mmax<-qnbinom(.9999,size=mu^2/sig2,prob=mu/(mu+sig2))
      npr<-dnbinom(0:mmax,size=mu^2/sig2,prob=mu/(mu+sig2))
   } else {
      mmax<-qpois(.9999,lambda=mu)
      npr<-dpois(0:mmax,lambda=mu)
   }
   return(npr/sum(npr))
}

findWparm<-function(mu,sig2){
  Ws2m2.a<-function(a,mu,sig2){
    return(((lgamma(1+2/a)-2*lgamma(1+1/a)-log(sig2/mu^2+1)))^2)
  }
  a <- optimize(Ws2m2.a, c(0, 1000), tol = 0.0001, mu = mu, sig2=sig2)$minimum
  b<-mu/gamma(1+1/a)
  return(c(a,b))
}
f.wei.a<-function(a,si,mu,rI){# objective function to minimize in finding Weibull alpha parameter given r and meanCP
   abs(1-integrate(pweibull,lower=0,upper=si,shape=a,scale=mu/gamma(1+1/a))$val/si-rI)
}
f.lnorm.a<-function(a,si,mu,rI){# objective function to minimize in finding Weibull alpha parameter given r and meanCP
   abs(1-integrate(plnorm,lower=0,upper=si,meanlog=log(mu)-a/2,sdlog=sqrt(a))$val/si-rI)
}
f.ll.a<-function(a,si,mu,rI){# objective function to minimize in finding Weibull alpha parameter given r and meanCP
   abs(1-integrate(pllogis,lower=0,upper=si,shape=a,scale=mu*a/pi*sin(pi/a))$val/si-rI)
}
f.e.a<-function(a,si,mu,rI){# objective function to minimize in finding Weibull alpha parameter given r and meanCP
   abs(1-integrate(pexp,lower=0,upper=si,rate=a)$val/si-rI)
}

# find alpha and beta parameters given r, CP, and Ir
abgrCP<-function(pd,r,Ir,CP){
   amax<-0
   if (pd=="Weibull"){
      CP<-max(CP,r*Ir/.99)
      for (a in 1:40){ #find an appropriate upper bound on alpha
         if(1-integrate(pweibull,lower=0,upper=Ir,shape=a,scale=CP/gamma(1+1/a))$val/Ir>r){
            amax<-a
            break
         }
      }
      a<-optimize(f=f.wei.a,si=Ir,mu=CP,rI=r,interval=c(0.01,amax))$min
      b<-CP/gamma(1+1/a)
   }  else if (pd=="Lognormal"){
      CP<-max(CP,r*Ir/.999)
      a<-seq(0.001,20,length=1000)
      imax<-2
      for (i in 1:length(a)){ #find an appropriate bounds on alpha
         if(1-integrate(plnorm,lower=0,upper=Ir,meanlog=log(CP)-a[i]/2,sdlog=sqrt(a[i]))$val/Ir<r){
            imax<-i
            break
         }
      }
      a<-optimize(f=f.lnorm.a,si=Ir,mu=CP,rI=r,interval=c(a[imax-1],a[imax]))$min
      b<-log(CP)-a/2
   } else if (pd=="Log-Logistic"){
      CP<-max(CP,r*Ir/.999)
      for (a in 1:40){ #find an appropriate upper bound on alpha
         if(1-integrate(pllogis,lower=0,upper=Ir,shape=a,scale=CP*sin(pi/a)*a/pi)$val/Ir>r){
            amax<-a
            break
         }
      }
      a<-optimize(f=f.ll.a,si=Ir,mu=CP,rI=r,interval=c(max(0.01,amax-1),amax))$min
      b<-CP*sin(pi/a)*(a/pi)
   } else if (pd=="Exponential"){
      a<-optimize(f=f.e.a,si=Ir,mu=CP,rI=r,interval=c(0.01,20))$min
      b<-1/a
   }
   return(c(a,b))
}
rCPgab<-function(pd,a,b,Ir){
   if (pd=="Weibull"){
      CP<-b*gamma(1+1/a)
      r<-1-integrate(pweibull,lower=0,upper=Ir,shape=a,scale=b)$val/Ir
   } else if (pd=="Lognormal") {
      CP<-exp(b+a/2)
      r<-1-integrate(plnorm,lower=0,upper=Ir,meanlog=b,sdlog=sqrt(a))$val/Ir
   } else if (pd=="Log-Logistic") {
      if (a>1) {
        CP<-b/a*pi/sin(pi/a)
      } else {
        CP<- -1
      }
      r<-1-integrate(pllogis,lower=0,upper=Ir,shape=a,scale=b)$val/Ir
   }
   if (pd=="Exponential"){
    CP<-b
    r<-1-integrate(pexp,lower=0,upper=Ir,rate=1/b)$val/Ir
   }
   return(c(CP,r))
}
bgar.obj<-function(b,pd,a,r,Ir){
  if (pd=="Weibull"){
    abs(1-integrate(pweibull,lower=0,upper=Ir,shape=a,scale=b)$val/Ir-r)
  } else if (pd=="Exponential"){
    1/a
  } else if (pd=="Lognormal"){
    abs(1-integrate(plnorm,lower=0,upper=Ir,sdlog=sqrt(a),meanlog=b)$val/Ir-r)
  } else if (pd=="Log-Logistic"){
    abs(1-integrate(pllogis,lower=0,upper=Ir,shape=a,scale=b)$val/Ir-r)
  }
}
bCPgar<-function(pd,a,r,Ir){
  if (pd=="Weibull"){
    intlim<-c(0.00005,10000)
    b<-optimize(bgar.obj,interval=intlim,pd=pd,a=a,r=r,Ir=Ir)$min
    CP<-b*gamma(1+1/a)
  } else if (pd=="Exponential") {
    a<-optimize(f=f.e.a,si=Ir,mu=CP,rI=r,interval=c(0.01,20))$min
    b<-1/a
    CP<-b
  } else if (pd=="Lognormal"){
    intlim<-c(-1.5,5.5)-a/2
    b<-optimize(bgar.obj,interval=intlim,pd=pd,a=a,r=r,Ir=Ir)$min
    CP<-exp(b+a/2)
  } else if (pd=="Log-Logistic"){
    intlim<-c(.05,100)
    b<-optimize(bgar.obj,interval=intlim,pd=pd,a=a,r=r,Ir=Ir)$min
    CP<-(b*pi/a)/sin(pi/a)
  }
   return(c(b,CP))
}
colr<-c(colors()[c(490,552,498,652)],"#FFFFFFBF")
breaks<-c(seq(0,0.05,len=200/5),seq(0.05,0.1,len=200/5),seq(0.1,.3,len=200/5),seq(0.3,.5,len=200/5),seq(0.5,1,len=200/5+1))

repmat <- function(X,m,n){
##R equivalent of repmat (matlab)
  mx <- dim(X)[1]
  nx <- dim(X)[2]
  matrix(t(matrix(X,mx,nx*n)),mx*m,nx*n,byrow=T)
}
