############
# pMgtA is a script file for determining probabilities P(M > tau) for given sets of parameters
# and draws a figure (or calls a function in base funcs.r to draw a figure).

# It is called by the Excel "Draw Graph" button on the "Design Tradeoffs" form. The Excel button
# reads the following parameters into R
# tau = Action threshold = number of fatalities that would trigger mitigation
# Ifix, ffix, phifix = variables to indicate whether the given parameter is fixed covers a range of values
# Imin, Imax = bounds on search intervals to consider (when Ifix = 0)
# si, span = search schedule parameters (when Ifix = 1)
# fmin, fmax = range of search efficiency values to consider (when ffix = 0)
# p = searcher proficiencey(when ffix = 1)
# phimin, phimax = range of search coverages to consider (when phifix = 0)
# A = search coverage (when phifix = 1)
# grinc = the number of subdivisions to consider for variable parameters, e.g., if ffix = 0, then f values to be considered are f<-seq(fmin, fmax, length = grinc)
# X = observed carcass count
# k = factor by which search efficiency changes with each successive search
# persistence_distn = "Weibull", "Exponential", "Lognormal", "Log-logistic"
# pda, pdb = alpha and beta parameters for persistence distribution
# arrfun = name of carcass arrival function through time = "Uniform", "Beta" (if arrfun == "Beta" then arrival function parameters afa and afb are also loaded into R)
# prior.f = name of prior distribution = "Negative Binomial", "Poisson", "Uniform", "User-defined", "Informed"
# pra, prb = parameters for prior distribution.
# prior.f = "Negative Binomial" ==> pda = r, pdb = p
# prior.f = "Poisson" ==> pda = rate
# prior.f = "Uniform" ==> pda = maximum M, pdb = 0
# prior.f = "User-defined" | "Informed" ==> pda = mean, pdb = sd (but these are for display purposes only...the actual distribution is read from Excel
# prior_M = list of prior probabilities P(M = 0), P(M = 1), ... in case prior.f = "User-defined" | "Informed"
############
# Priors must be stored as a list of probabilities. The far talis of the distributions are clipped.
if (prior.f=="Uniform"){
   cut_off<-pra
   prior_M<-rep(1/(pra+1),pra+1)
} else if (prior.f=="User-defined" | prior.f=="Informed"){
   cut_off<-length(prior_M)-1
  # assumes prior_M has already been read into R
}
if (Ifix==0){
   ni<-(span/Imin):(span/Imax)
   nsi<-length(ni)
   si<-span/ni
   sitest<-floor(si/Imin)*Imin
   sitest<-c(0,sitest)
   si<-si[sitest[2:nsi]-sitest[1:(nsi-1)]>0]
   nsi<-length(si)
   if (nsi>grinc){
      si<-si[c(1,1+which(diff(round((1:nsi)*(grinc-2)/nsi))>0),nsi)]
      nsi<-length(si)
   }
} else {
   days<-seq(0,span,by=si)
   if (span/si-floor(span/si)>.8){
      days<-c(days,span)
   }
   nt<-length(days)-1
   if (arrfun=="Uniform"){
      parrt<-numeric(nt)+1/nt
   } else if (arrfun=="Beta"){
      dug<-pbeta(days/span,shape1=afa,shape2=afb)
      parrt<-dug[2:(nt+1)]-dug[1:nt]
   }
}
if (ffix==0){
   f<-seq(fmin,fmax,length=grinc)
}
if (phifix==0){
   phi<-seq(phimin,phimax,length=grinc)
}
if (phifix==0 & ffix==0) {
   fphi<-f%o%phi
} else {
   fphi<-f*phi
}

#nt<-length(days)-1
if (arrfun=="Uniform"){
   fn.arrivals <- function(tt){
      return(tt^0/span)
   }
} else if (arrfun=="Beta"){
   fn.arrivals <- function(x){
      return(dbeta(x/span,shape1=afa,shape2=afb)/span)
   }
}
if (phifix+ffix+Ifix>0 & dev.cur()>1){
   if (par("mfrow")!=c(1,1)){
      dev.off()
      par(mar=c(4,4,4,.5))
   }
}
if (ffix==0 & phifix==0 &  Ifix == 1){
   pMgtA_fphi(si=si,sivar=FALSE)
} else if (ffix==1 & phifix==0 &  Ifix==0){
   pMgtA_phiI()
} else if (ffix==0 & phifix==1 & Ifix==0){
   pMgtA_fI()
} else if (ffix==1 & phifix==1 & Ifix==1){
   pM<-Mest1B(X=X, phi=phi, f=f, k=k, pda=pda, pdb=pdb, afa=afa, afb=afb, pra=pra, prb=prb, persistence_distn=persistence_distn, fn.arrivals=fn.arrivals, prior.f=prior.f, days=days)
   pMemgX<-pM$'P(M=m)'
   pobs<-pM$'P(Observe|Arrive)'
   plotPost(vary=FALSE)
   if (length(pMemgX)<A+1){
      Pex<-0
   } else {
      Pex<-signif(1-sum(pMemgX[1:(A+1)]),3)
   }
#   mtext(paste("Threshold: tau = ", A,",   P(M > tau) = ",Pex,sep=""))             ###%%%
   mtext(paste("Threshold: \u03c4 = ", A,",   P(M > \u03c4) = ",Pex,sep=""))
} else if (ffix==1 & phifix==1 & Ifix==0){
   pMgtA_I()
} else if (ffix==0 & phifix==1 & Ifix==1){
   pMgtA_f()
} else if (ffix==1 & phifix==0 & Ifix==1){
   pMgtA_phi()
} else if (ffix==0 & phifix==0 & Ifix==0){ #all are variable, so draw five graphs for five different si values in a 2 x 3 array of graphs
   partial=TRUE
   win.graph(width=9,height=6)
#   par(mfrow=c(2,3),family="serif")
   par(mfrow=c(2,3),family="sans")
   si<-unique(ceiling(((Imax/Imin)^.25)^(0:4)*Imin))    # this gives integer search intervals but some have awful endpoints
   si[length(si)]<-Imax
   nsi<-length(si)
   for (si.ind in 1:nsi){
      #set 'days'
      days<-seq(0,span,by=si[si.ind])
      nt<-length(days)-1
      days[nt+1]<-span
      if (arrfun=="Uniform"){
         parrt<-numeric(nt)+1/nt
      } else if (arrfun=="Beta"){
         dug<-pbeta(days/span,shape1=afa,shape2=afb)
         parrt<-dug[2:(nt+1)]-dug[1:nt]
      }
   #draw graph
   pMgtA_fphi(si=si[si.ind],sivar=TRUE)
   }
   plot(0,0,xlim=c(0,1),ylim=c(0,1),type='n',axes=F,xlab='',ylab='',xaxs='i',yaxs='i')
   bringToTop()
   text(0,.95,paste("P(M > ",A,"| X = ",X,")",sep=""),cex=1.5,adj=0)
   arl<-paste("Arrivals:\n   ",arrfun)
   if (arrfun=="Uniform"){
      arrlab<-arl
   } else if (arrfun=="Beta"){
      arrlab<-substitute(list(.arl, alpha==.afa,beta==.afb),list(.arl=arl,.afa=afa,.afb=afb))
   }
   text(0,.72,arrlab,adj=0)
   pdl<-paste("Persistence:\n   ",persistence_distn)
   if (persistence_distn=="Exponential"){
      pdlab1<-substitute(list(.pdl,lambda == .lambda),list(.pdl=pdl,.lambda=pda))
      pdlab2<-paste("    mean CP = ",round(1/pda,2),", median = ", round(qexp(.5,rate=pda),2),sep='')
   } else if (persistence_distn=="Weibull"){
      pdlab1<-substitute(list(.pdl,alpha == .alpha,beta==.beta),list(.pdl=pdl,.alpha=pda,.beta=pdb))
      pdlab2<-paste("    mean CP = ",round(pdb*gamma(1+1/pda),2),", median = ",round(qweibull(.5,pda,pdb),2),sep='')
   } else if (persistence_distn=="Lognormal"){
      pdlab1<-substitute(list(.pdl,alpha == .alpha,beta==.beta),list(.pdl=pdl,.alpha=pda,.beta=pdb))
      pdlab2<-paste("    mean CP = ", round(exp(pdb+pda/2),2),", median = ",round(qlnorm(.5,meanlog=pdb,sdlog=sqrt(pda)),2),sep='')
   } else if (persistence_distn=="Log-Logistic"){
      pdlab1<-substitute(list(.pdl,alpha == .alpha,beta==.beta),list(.pdl=pdl,.alpha=pda,.beta=pdb))
      pdlab2<-paste("    mean CP = ", round(pdb*pi/pda/sin(pi/pda),2),", median = ",round(qllogis(.5,shape=pda,scale=pdb),2),sep='')
   }
   text(0,.52,pdlab1,adj=0)
   text(0,.44,pdlab2,adj=0)
   prl<-paste("Prior:\n   ",prior.f)
   if (prior.f=="Uniform"){
      prlab<-paste(prl,", max = ", pra,sep='')
   } else if (prior.f=="User-defined"){
      prlab<-paste(prl, ", 95th percentile = ",pra,sep='')
   } else if (prior.f=="Informed"){
      prlab<-paste(prl, ", 95th percentile = ",pra,sep='')
   }
   text(0,.27,prlab,adj=0)
   text(0,.08,paste("span = ",span, "\nk = ",k), adj=0)
}